/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
/** @file SprObject.h
	s^NX̊{NX
 */
#ifndef SPR_OBJECT_H
#define SPR_OBJECT_H
#include "../SprBase.h"
#include <iosfwd>

namespace Spr{;

class Object;

#if !defined SWIG

#undef DCAST
#define DCAST(T,p) DCastImp((T*)0, p)

///	^ւ̃LXg
///	ÓIϊ\ȏꍇ If->Obj ̃LXg
template <class TO, class FROM> TO* DCastImp(TO* dmmy, FROM* p){
	return TO::GetSelf(p);
}
template <class TO, class FROM> UTRef<TO> DCastImp(TO* dmmy, UTRef<FROM> p){
	return TO::GetSelf(&*p);
}

#undef XCAST
#define XCAST(p) CastToXCastPtr(p)
///	C^tF[XNX̃LXg
template <class T>
class XCastPtr{
public:
	template <class X> operator X*() const {
		T* t = (T*) this;
		return DCastImp((X*)0, t);
	}
	template <class X> operator UTRef<X>() const {
		T* t = (T*) this;
		return DCastImp((X*)0, t);
	}
};

template <class T> XCastPtr<T>& CastToXCastPtr(const T* ptr){
	return *(XCastPtr<T>*)(void*)ptr;
}
template <class T> XCastPtr<T>& CastToXCastPtr(const UTRef<T> ptr){
	return *(XCastPtr<T>*)(void*)&*ptr;
}


struct ObjectIf;
class IfInfo;
class UTTypeDesc;

/**	Factory (IuWFNg𐶐NX)̊{NX
*/
class FactoryBase:public UTRefCount{
public:
	virtual ~FactoryBase(){}
	///	쐬IfInfo
	virtual const IfInfo* GetIfInfo() const =0;
	///	iĩIuWFNg desc  parent̎qƂĐD
	virtual ObjectIf* Create(const void* desc, ObjectIf* parent) =0;
};

/**	C^tF[X̌^NX̊{NXDNXp֌WĂāCDCAST()ȂǂpD
1̃C^tF[XNXɕt1̃CX^XłD	*/
class IfInfo: public UTRefCount{
public:
	///	IfInfoid NXRD
	int id;
	///	id̍ől
	static int maxId;
	///	desc^ɂĂ̏ڍׂȏ
	UTTypeDesc* desc;
	///	NX
	const char* className;
	///	{NX
	IfInfo** base;

	typedef std::vector< UTRef<FactoryBase> > Factories;
	///	qIuWFNg𐶐NX(t@Ng[)
	Factories factories;
	///	RXgN^
	IfInfo(const char* cn, IfInfo** b);
	///	fXgN^
	virtual ~IfInfo() {};
	///	NX
	virtual const char* ClassName() const =0;
	///	key pĂ邩ǂ`FbN
	bool Inherit(const char* key) const;
	///	key pĂ邩ǂ`FbN
	bool Inherit(const IfInfo* key) const;
	///	t@Ng(IuWFNg𐶐NX)̓o^
	void RegisterFactory(FactoryBase* f) const ;
	///	w(info)̃IuWFNgt@Ng
	FactoryBase* FindFactory(const IfInfo* info) const;
	///
	static IfInfo* Find(const char* cname);
};
///	IfInfo̎D1NX1CX^X
template <class T>
class IfInfoImp: public IfInfo{
public:
	IfInfoImp(const char* cn, IfInfo** b): IfInfo(cn, b){}
	virtual const char* ClassName() const { return className; }
	virtual void* CreateInstance() const{ return 0;}
};

///	C^tF[Xׂo̐錾D
#define IF_DEF_FOR_OBJECTIF(cls)										\
public:																	\
	static IfInfoImp<cls##If> ifInfo;	/*	If̌^	*/				\
	virtual const IfInfo* GetIfInfo() const {							\
		return GetIfInfoStatic();										\
	}																	\
	static const IfInfo* SPR_CDECL GetIfInfoStatic();					\
	static void* operator new(size_t) {									\
        assert(0);	/*	Don't allocate interfaces	*/					\
        return (void*)1;												\
    }																	\
	static void operator delete(void* pv) { /*	nothing	to do */ }		\
	/*	? -> If	*/														\
	template <class O>													\
	static cls##If* GetSelf(const O * p){								\
		return p ? p->GetIf((cls##If*)0) : NULL;						\
	}																	\
	/*	^If -> If	*/												\
	cls##If* GetIf(cls##If* i) const { return (cls##If*)this; }			\
	/*	݃LXg	*/												\
	XCastPtr<cls##If>& Cast() const{									\
		return *(XCastPtr<cls##If>*)(void*)this; }						\
	/*	ٌ^If -> Ifւ̌^ϊ */										\
	template <class I> I* GetIf(I*) const {									\
		if (GetIfInfo()->Inherit(I::GetIfInfoStatic())) return (I*)this;	\
		return (I*)GetIfDynamic(I::GetIfInfoStatic());						\
	}																		\

///	fBXNv^ׂo̐錾D
#define DESC_DEF_FOR_OBJECT(cls)										\
public:																	\
	const static IfInfo* GetIfInfo(){									\
		return cls##If::GetIfInfoStatic();								\
	}																	\

#endif // !SWIG

// RubyȂǂ̃|[gŎgpLXg
#define IF_HLANG_CAST(cls) static cls##If* Cast(ObjectIf* o){return DCAST(cls##If, o);}

#ifdef SWIG
#define IF_DEF(cls)	IF_HLANG_CAST(cls)
#else
#define IF_DEF(cls)	IF_DEF_FOR_OBJECTIF(cls) IF_HLANG_CAST(cls)
#endif

///	ׂẴC^tF[XNX̊{NX
struct ObjectIf{
	IF_DEF_FOR_OBJECTIF(Object);
	virtual ~ObjectIf();
	///	IuWFNg擾
	virtual Object* GetObj(const UTTypeInfo* info) const =0;
	///	IȃC^tF[X̎擾
	virtual ObjectIf* GetIfDynamic(const IfInfo* info) const = 0;
	///	fobOp̕\BqIuWFNg܂ށB
	virtual void Print(std::ostream& os) const =0;	
	///	fobOp̕\BqIuWFNg܂܂ȂB
	virtual void PrintShort(std::ostream& os) const =0;	

	///	@name QƃJE^֌W
	//@{
	///
	virtual int AddRef()=0;
	///
	virtual int DelRef()=0;
	///
	virtual int RefCount()=0;
	//@}


	///	@name qIuWFNg
	//@{
	///	qIuWFNg̐
	virtual size_t NChildObject() const =0;
	///	qIuWFNg̎擾
	virtual ObjectIf* GetChildObject(size_t pos) = 0;
	virtual const ObjectIf* GetChildObject(size_t pos) const = 0;
	/**	qIuWFNg̒ǉD̃IuWFNg̎qIuWFNgƂĒǉĂ悢D
		Ⴆ΁CGRFrame̓c[邪CSm[hGRScene̎qłD*/
	virtual bool AddChildObject(ObjectIf* o)=0;
	///	qIuWFNg̍폜
	virtual bool DelChildObject(ObjectIf* o)=0;
	///	ׂĂ̎qIuWFNg̍폜ƁAvpeB̃NA
	virtual void Clear()=0;
	///	IuWFNg쐬CAddChildObject()ĂԁD
	virtual ObjectIf* CreateObject(const IfInfo* info, const void* desc)=0;
	///	CreateObjectĂяo[eBeB֐
	template <class T> ObjectIf* CreateObject(const T& desc){
		return CreateObject(desc.GetIfInfo(), &desc);
	}
	//@}

	///	@name fXNv^Ə
	//@{
	/**	fXNv^̓ǂݏo(QƔ NULLԂƂ)D
		ꂪĂȂĂCObjectIf::GetDesc()͎Ă邱ƂD	*/
	virtual const void* GetDescAddress() const = 0;
	/**	fXNv^̓ǂݏo(Rs[ s(falseԂ)Ƃ)D
		ObjectIf::GetDescAddress() ĂȂĂC͎Ă邱ƂD	*/
	virtual bool GetDesc(void* desc) const = 0;
	/** fXNv^̐ݒ */
	virtual void SetDesc(const void* desc) = 0;
	/**	fXNv^̃TCY	*/
	virtual size_t GetDescSize() const = 0;
	/**	Ԃ̓ǂݏo(QƔ NULLԂƂ)D
		ꂪĂȂĂCObjectIf::GetState()͎Ă邱ƂD	*/
	virtual const void* GetStateAddress() const = 0;
	/**	Ԃ̓ǂݏo(Rs[ s(falseԂ)Ƃ)D
		ObjectIf::GetStateAddress() ĂȂĂC͎Ă邱ƂD	*/
	virtual bool GetState(void* state) const = 0;
	/**	Ԃ̐ݒ	*/
	virtual void SetState(const void* state) = 0;
	/**	Ԃ̃TCY	*/
	virtual size_t GetStateSize() const = 0;
	/**	ubNԌ^ɏ	*/
	virtual void ConstructState(void* m) const = 0;
	/**	Ԍ^ubNɖ߂	*/
	virtual void DestructState(void* m) const = 0;
	//@}
};

///	C^tF[XNXւ̃|C^̔z
struct ObjectIfs
#if !defined SWIG	
	: public UTStack<ObjectIf*>
#endif
{
	/*void PrintShort(std::ostream& os) const{
		for(const_iterator it = begin(); it!=end(); ++it){
			(*it)->PrintShort(os);
		}
	}
	void Print(std::ostream& os) const{
		for(const_iterator it = begin(); it!=end(); ++it){
			(*it)->Print(os);
		}
	}*/
	typedef UTStack<ObjectIf*> container_t;
	void Push(ObjectIf* obj){container_t::Push(obj);}
	void Pop(){container_t::Pop();}
	ObjectIf* Top(){return container_t::Top();}
};

struct NameManagerIf;

///	OIuWFNg̃C^tF[X
struct NamedObjectIf: public ObjectIf{
	IF_DEF(NamedObject);
	///	O̎擾
	virtual const char* GetName() const =0;
	///	O̐ݒ
	virtual void SetName(const char* n) =0;
	///	OǗIuWFNg̎擾
	virtual NameManagerIf* GetNameManager() =0;
};

struct SceneIf;
///	V[Ot\m[h̃C^tF[X
struct SceneObjectIf: NamedObjectIf{
	IF_DEF(SceneObject);
	///	Scene̎擾
	virtual SceneIf* GetScene() =0;
};


///	IuWFNgc[̏ԂɕۑĂNX
struct ObjectStatesIf: public ObjectIf{
	IF_DEF(ObjectStates);

	///	oƂ̎qZ[u邽߂ɕKvȃmۂD
	virtual void AllocateState(ObjectIf* o) = 0;
	///	Ԃ̃
	virtual void ReleaseState(ObjectIf* o) = 0;
	///	Ԃ̃TCY߂
	virtual size_t CalcStateSize(ObjectIf* o) = 0;

	///	ԂZ[uD
	virtual void SaveState(ObjectIf* o) = 0;
	///	Ԃ[hD
	virtual void LoadState(ObjectIf* o) = 0;
};
ObjectStatesIf* SPR_CDECL CreateObjectStates();

}

#endif
