/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef SPR_GRRENDER_H
#define SPR_GRRENDER_H

#include <float.h>
#include <Physics/SprPHSolid.h>
#include <Physics/SprPHScene.h>
#include <Collision/SprCDShape.h>
#include <Graphics/SprGRFrame.h>
#include <Graphics/SprGRVertex.h>
#include <Graphics/SprGRShader.h>
namespace Spr{;

/**	\addtogroup	gpGraphics	*/
//@{

/**
 *	@file SprGRRender.h
 *	@brief OtBbNX_[̊{NXA{C^tF[X
 */

/**	@brief	eLXg`̃tHg */
class SPR_DLL GRFont{
public:
	int height;				///<	tHg̍
	int width;				///<	ϕ
	int weight;				///<	tHg̑i0 - 900j
	std::string face;		///<	^CvtFCX
	unsigned long color;	///<	tHg̐F
	bool bItalic;			///<	C^bN
	GRFont(int h=20, const char* f=NULL){
		height=h; width=0;
		face=f ? f : "";
		weight = 400; color=0xffffffff; bItalic=false;
	}
	void SetColor(unsigned long c){ color = c; }
	bool operator < (GRFont& f){
		if (face < f.face) return true;
		if (height < f.height) return true;
		if (color < f.color) return true;
		return false;
	}
	GRFont& operator=(const GRFont& rhs) {
		if (this==&rhs)	return *this;
		height	= rhs.height;
		width	= rhs.width;
		weight	= rhs.weight;
		face	= rhs.face;
		color	= rhs.color;
		bItalic	= rhs.bItalic;
		return *this;
	}
};
	
/**	@brief			*/
struct GRLightDesc : GRVisualDesc{
    Vec4f ambient;		///<	
    Vec4f diffuse;		///<	gU
    Vec4f specular;		///<	ʌ
    Vec4f position;		///<	ʒuiw=1.0œ_Aw=0.0ŕsj
    float range;		///<	͈͂́ideviceDirectX̏ꍇ̂ݗp\j
	/**
	 *	@name	p[^
	 *@@ ݒ肷邱ƂŌ̌\łB \n
	 *@@ ̃p[^ݒ肷ƁA痣قǍɂʂ͏ȂB \n
	 *@@ ftHgł́AȂɐݒ肳ĂB \n
	 *@@ s̏ꍇA͂ȂB \n
	 *@@@@ W = 1/( att0 + att1 * d + att2 * d^2)@@@d:
	 *	@{
	 */
	float attenuation0;	///<	茸
    float attenuation1;	///<	`
    float attenuation2;	///<	2
	/** @} */	// end of p[^

	Vec3f spotDirection;///<	X|bgCǧ
    float spotFalloff;	///<	X|bgCg̉~ł̌(傫قǋ}s) 0..
	float spotInner;	///<	X|bgCg̒S(R[)ideviceDirectX̏ꍇ̂ݗp\j 0..spotCutoff
	float spotCutoff;	///<	X|bgCg̍Lpx(x)(OR[) 0..(pi)
	GRLightDesc(){
		type = LIGHT;
		ambient = Vec4f(0.0, 0.0, 0.0, 1.0);
		diffuse = Vec4f(1.0, 1.0, 1.0, 1.0);
		specular = Vec4f(1.0, 1.0, 1.0, 1.0);
		position = Vec4f(0.0, 0.0, 1.0, 1.0);
		range = FLT_MAX;
		attenuation0 = 1.0f;
		attenuation1 = 0.0f;
		attenuation2 = 0.0f;
		spotDirection = Vec3f(0.0, -1.0, 0.0);
		spotFalloff  = 0.0f;
		spotInner    = 0.0f;
		spotCutoff   = 180.0f;
	}
};

/** @brief ̊{NX		*/
struct GRLightIf: public GRVisualIf{
	IF_DEF(GRLight);
};

/**	@brief	ގ	*/
struct GRMaterialDesc : GRVisualDesc{
	Vec4f ambient;					///<	ɑ΂锽˗
	Vec4f diffuse;					///<	gUɑ΂锽˗
	Vec4f specular;					///<	ʌɑ΂锽˗
	Vec4f emissive;					///<	ˋPx
	float power;					///<	ʔ˂̋xAʌW
	std::string	texname;			///<	eNX`t@C

	GRMaterialDesc(){
		type = MATERIAL;
		ambient = Vec4f(0.2, 0.2, 0.2, 1.0);
		diffuse = Vec4f(0.8, 0.8, 0.8, 1.0);
		specular = Vec4f(1.0, 1.0, 1.0, 1.0);
		emissive = Vec4f(0.0, 0.0, 0.0, 1.0);
		power = 20.0;
	}
	GRMaterialDesc(Vec4f a, Vec4f d, Vec4f s, Vec4f e, float p):
		ambient(a), diffuse(d), specular(s), emissive(e), power(p){}
	GRMaterialDesc(Vec4f c, float p):
		ambient(c), diffuse(c), specular(c), emissive(c), power(p){}
	GRMaterialDesc(Vec4f c):
		ambient(c), diffuse(c), specular(c), emissive(c), power(0.0f){}
	/**	W()vf́AAt@l(0.0`1.0œx\). 1.0s\.
		materialW()vf𔻒肵āAŝÂ𔻒肷. 
		ȃIuWFNg`ƂÂ̂珇Ԃɕ`悵ȂƁAӐ}ɔʂƂȂ. */
	bool IsOpaque() const {		
		return ambient.W() >= 1.0 && diffuse.W() >= 1.0 && specular.W() >= 1.0 && emissive.W() >= 1.0;
	}	
};

/** @brief@ގ̊{NX@@	*/
struct GRMaterialIf: public GRVisualIf{
	IF_DEF(GRMaterial);
	virtual bool IsOpaque() const = 0;
};

/**	@brief	J̏			*/
struct GRCameraDesc : GRVisualDesc{
	Vec2f size;				///<	XN[̃TCY
	Vec2f center;			///<	J̃XN[̂
	float front, back;		///<	_Nbvʂ܂ł̑΋i̒lŎwj
	//GRCameraDesc():center(Vec2f()), size(Vec2f(0.2f, 0)), front(0.1f), back(500.0f){}
	GRCameraDesc(Vec2f sz = Vec2f(0.2f, 0.0f), Vec2f c = Vec2f(), float f = 0.1f, float b = 500.0f):
		size(sz), center(c), front(f), back(b) { type = CAMERA;}
};

struct GRFrameIf;

struct GRCameraIf: public GRVisualIf{
	IF_DEF(GRCamera);
	virtual GRFrameIf* GetFrame()=0;
	virtual void SetFrame(GRFrameIf* fr)=0;
};

struct GRDeviceIf;
typedef unsigned GRHandler;

/**	@brief	OtBbNX_[̊{NXi[UC^tF[Xj */
struct GRRenderBaseIf: public ObjectIf{
	IF_DEF(GRRenderBase);

	///	v~eBu̎
	enum TPrimitiveType {
		POINTS,
		LINES,
		LINE_STRIP,
		TRIANGLES,
		TRIANGLE_STRIP,
		TRIANGLE_FAN,
		QUADS
	};
	/**
	 *	@name	fvXobt@@ɗp锻
	 *	@{
	 */
	enum TDepthFunc{
		DF_NEVER,		///<	VZlɊ֌WȂXVȂ
		DF_LESS,		///<	VZl΍XVidefault)
		DF_EQUAL,		///<	VZlΕ`
		DF_LEQUAL,		///<	VZl傫ȂΕ`
		DF_GREATER,		///<	VZl傫Ε`
		DF_NOTEQUAL,	///<	VZlƓȂΕ`
		DF_GEQUAL,		///<	VZlȂΕ`
		DF_ALWAYS		///<	VZlɊ֌WȂXV
	};
	/** @} */	
	/**
	 *	@name	At@ufBO̍W
	 *@@ SRCꂩ`悳鍇FADESTłɕ`ꂽJ[obt@̐F    \n
	 *@@@@  =SRC * SRC̃ufBOW + DEST * DEST̃ufBOW 			\n
	 *@@ ABF_BOTHINVSRCALPHAABF_BLENDFACTORABF_INVBLENDFACTOR ɊւẮA			\n
	 *@@ deviceDirectX̂Ƃ̂ݗp\Bw肵ĂBF_ZEROƂĈB
	 *	@{
	 */
	enum TBlendFunc{
		BF_ZERO,				///<	(0,0,0,0)	
		BF_ONE,					///<	(1,1,1,1)
		BF_SRCCOLOR,			///<	(Rs, Gs, Bs, As)
		BF_INVSRCCOLOR,			///<	(1-Rs, 1-Gs, 1-Bs, 1-As) 
		BF_SRCALPHA,			///<	(As, As, As, As) 
		BF_INVSRCALPHA,			///<	(1-As, 1-As, 1-As, 1-As)
		BF_DESTALPHA,			///<	(Ad, Ad, Ad, Ad)  
		BF_INVDESTALPHA,		///<	(1-Ad, 1-Ad, 1-Ad, 1-Ad) 
		BF_DESTCOLOR,			///<	(Rd, Gd, Bd, Ad)
		BF_INVDESTCOLOR,		///<	(1-Rd, 1-Gd, 1-Bd, 1-Ad) 
		BF_SRCALPHASAT,			///<	(f, f, f, 1)Af = min(As, 1-Ad) 
		BF_BOTHINVSRCALPHA,		///<	SRCufBOW(1-As, 1-As, 1-As, 1-As)BDSTufBOW(As, As, As, As)B
		BF_BLENDFACTOR,			///<	t[obt@Ɏg萔Fō
		BF_INVBLENDFACTOR		///<	t[obt@Ɏg萔F𔽓]Fō
	};
	/** @} */

	///	r[|[g̐ݒ
	virtual void SetViewport(Vec2f pos, Vec2f sz)=0;
	///	obt@NA
	virtual void ClearBuffer()=0;
	///	_O̊JnOɌĂԊ֐
	virtual void BeginScene()=0;
	///	_ȌIɌĂԊ֐
	virtual void EndScene()=0;
	///	Jg̎_safvŒu
	virtual void SetViewMatrix(const Affinef& afv)=0;
	///	Jg̓esafpŒu
	virtual void SetProjectionMatrix(const Affinef& afp)=0;
	///	Jg̓es擾
	virtual void GetProjectionMatrix(const Affinef& afp)=0;
	///	Jg̃fsafwŒu
	virtual void SetModelMatrix(const Affinef& afw)=0;
	///	Jg̃fsɑ΂afw|
	virtual void MultModelMatrix(const Affinef& afw)=0;
	///	Jg̃fsfsX^bN֕ۑ
	virtual void PushModelMatrix()=0;
	///	fsX^bNoAJg̃fsƂ
	virtual void PopModelMatrix()=0;
	/// uhϊs̑Svf폜
	virtual void ClearBlendMatrix()=0;
	/// uhϊsݒ肷
	virtual bool SetBlendMatrix(const Affinef& afb, unsigned int id=0)=0;
	///	_tH[}bg̎w
	virtual void SetVertexFormat(const GRVertexElement* e)=0;
	///	_VF[_[̎w	APIDȂǗv 2006.6.7 hase
	virtual void SetVertexShader(void* shader){}
	///	_w肵ăv~eBu`
	virtual void DrawDirect(GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0)=0;
	///	_ƃCfbNXw肵ăv~eBu`
	virtual void DrawIndexed(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0)=0;
 	///	_̐Ƃ̔zw肵āCv~eBu`
	virtual void DrawArrays(GRRenderBaseIf::TPrimitiveType ty, GRVertexArray* arrays, size_t count){}
 	///	CfbNXƒ_̐Ƃ̔zw肵āCv~eBu`
	virtual void DrawArrays(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, GRVertexArray* arrays, size_t count){}	
	///	_CNg`ɂ DiplayList ̍쐬
	virtual int CreateList(GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0)=0;
	virtual int CreateList(GRMaterialIf* mat, unsigned int texid, 
						   GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0)=0;
	/// IuWFNg DisplayList ̍쐬
	virtual int CreateList(float radius, int slices, int stacks)=0;
	virtual int CreateList(GRMaterialIf* mat, float radius, int slices, int stacks)=0;
	///	CfbNX`ɂ DiplayList ̍쐬
	virtual int CreateIndexedList(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0)=0;
	virtual int CreateIndexedList(GRMaterialIf* mat, 
								  GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0)=0;
	///	CfbNX`ɂVF[_Kp DisplayList ̍쐬iSetVertexFormat()  SetShaderFormat() ̌ɌĂԁj
	virtual int CreateShaderIndexedList(GRHandler shader, void* location, 
									GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0)=0;	
	///	DisplayList̕\
	virtual void DrawList(int i)=0;
	///	DisplayList̉
	virtual void ReleaseList(int i)=0;
	///	2eLXg̕`@@ Windows(VC)ł̂fontT|[gÅłfontw肵ĂpȂB
	virtual void DrawFont(Vec2f pos, const std::string str, const GRFont& font=0)=0;
	///	3eLXg̕`@@ Windows(VC)ł̂fontT|[gÅłfontw肵ĂpȂB	
	virtual void DrawFont(Vec3f pos, const std::string str, const GRFont& font=0)=0;
	///	`̍ގ̐ݒ
	virtual void SetMaterial(const GRMaterialDesc& mat)=0;
	virtual void SetMaterial(const GRMaterialIf* mat)=0;
	///	`悷_Ȇ̐ݒ
	virtual void SetLineWidth(float w)=0;
	///	X^bNPush
	virtual void PushLight(const GRLightDesc& light)=0;
	virtual void PushLight(const GRLightIf* light)=0;
	///	X^bNPop
	virtual void PopLight()=0;
	///	fvXobt@ւ݂̏/֎~
	virtual void SetDepthWrite(bool b)=0;
	///	fvXeXgL/ɂ
	virtual void SetDepthTest(bool b)=0;
	///	fvXobt@@ɗp锻w肷
	virtual void SetDepthFunc(GRRenderBaseIf::TDepthFunc f)=0;
	/// At@ufBOL/ɂ
	virtual void SetAlphaTest(bool b)=0;
	///	At@ufBÕ[hݒ(SRC̍W, DESTW)
	virtual void SetAlphaMode(GRRenderBaseIf::TBlendFunc src, GRRenderBaseIf::TBlendFunc dest)=0;
	/// eNX`̃[hi߂lFeNX`IDj
	virtual unsigned int LoadTexture(const std::string filename)=0;
	/// VF[_̏
	virtual void InitShader()=0;
	/// VF[_tH[}bg̐ݒ
	virtual void SetShaderFormat(GRShaderFormat::ShaderType type)=0;
	/// VF[_IuWFNg̍쐬
	virtual bool CreateShader(std::string vShaderFile, std::string fShaderFile, GRHandler& shader)=0;
	/// VF[_IuWFNg̍쐬AGRDeviceGL::shaderւ̓o^i炩ShaderFileo^ĂKvj	
	virtual GRHandler CreateShader()=0;
	/// VF[_̃\[XvOɓǂݍ݁AVF[_IuWFNgƊ֘At
	virtual bool ReadShaderSource(GRHandler shader, std::string file)=0;	
	/// P[V̎擾iSetShaderFormat()ŃVF[_tH[}bgݒ肵ĂKvj
	virtual void GetShaderLocation(GRHandler shader, void* location)=0;	
};

/**	@brief	OtBbNX_[̊{NXifoCX̐ݒAJ̐ݒj */
struct GRRenderIf: public GRRenderBaseIf{
	IF_DEF(GRRender);
	///	foCX̐ݒ
	virtual void SetDevice(GRDeviceIf* dev)=0;
	///	J̐ݒ
	virtual void SetCamera(const GRCameraDesc& cam)=0;
	///	XN[(EBhE)TCYύXViewportƎˉesݒ
	virtual void Reshape(Vec2f pos, Vec2f screenSize)=0;
};

/**	@brief	OtBbNX_[̃foCXNXDOpenGLDirectX̃bp */
struct GRDeviceIf: public GRRenderBaseIf{
	IF_DEF(GRDevice);
	///	
	virtual void Init()=0;
	///	fobOp̏ԃ|[g
	virtual void Print(std::ostream& os) const=0;
};

/**	@brief	OpenGL̃_[{NX */
struct GRDeviceGLIf: public GRDeviceIf{
	IF_DEF(GRDeviceGL);
};

/**	@brief	DirectX̃_[{NX */
/*
struct GRDeviceD3DIf: public GRDeviceIf{
	IF_DEF(GRDeviceD3D);
};
*/

/**	@brief	fobO񃌃_[̊{NX */
struct GRDebugRenderIf:public GRRenderIf{
	IF_DEF(GRDebugRender);
	///	_[ŗpӂĂގ(24)
	enum TMaterialSample {
		RED,
		GREEN,
		BLUE,
		YELLOW,
		CYAN,
		MAGENTA,
		WHITE,
		GRAY,
		ORANGE,
		BLOWN,
		LIGHT_BLUE,
		MIDIUM_PURPLE,
		DARK_GREEN,
		DARK_VIOLET,
		DARK_CYAN,
		GREEN_YELLOW,
		LIME_GREEN,
		INDIAN_RED,
		INDIGO,
		GREEN_INDIGO,
		OLIVE_GREEN,
		NAVY_BLUE,
		TURQUOISE_BLUE,
		EMERALD_GREEN
	};

	/// V[̑SẴIuWFNg_O
	virtual void DrawScene(PHSceneIf* scene)=0;
	/// ̂_O
	virtual void DrawSolid(PHSolidIf* so)=0;
	/// ʂ_O
	virtual void DrawFace(CDFaceIf* face, Vec3f * base)=0;
	/// w肵ގ(}eATv)蓖Ă
	virtual void SetMaterialSample(GRDebugRenderIf::TMaterialSample matname)=0;
};

//@}


}
#endif
