/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */

/** \addtogroup gpPhysics 	*/
//@{

/**
 *	@file SprPHScene.h
 *	@brief V[
*/
#ifndef SPR_PHSCENEIF_H
#define SPR_PHSCENEIF_H
#include <Foundation/SprScene.h>
#include <Physics/SprPHSolid.h>
#include <Physics/SprPHJoint.h>

namespace Spr{;

struct PHSdkIf;

struct CDShapeIf;
struct CDShapeDesc;

typedef PHConstraintDesc PHJointDesc;

//class PHConstraintEngine;
//class PHGravityEngine;

/// GW̃V[̏
struct PHSceneState{
	/// ϕXebv
	double timeStep;
	/// ϕ
	unsigned count;
	PHSceneState(){Init();}
	void Init(){
		timeStep = 0.005;
		count = 0;
	}
};
/// GW̃V[̃fXNv^
struct PHSceneDesc: public PHSceneState{
	///	ڐG̃[h
	enum ContactMode{
		MODE_NONE,		///< 
		MODE_PENALTY,	///< yieB@ŉ
		MODE_LCP		///< LCPŉ
	};
	
	Vec3f	gravity;	///< d͉xxNgDftHgl(0.0f, -9.8f, 0.0f)D
	int		numIteration;	///< LCP̔

	PHSceneDesc(){Init();}
	void Init(){
		gravity=Vec3f(0,-9.8f,0);
		numIteration = 15;
	}
};

///	V[
struct PHSceneIf : public SceneIf{
public:
	IF_DEF(PHScene);

	/** @brief SceneSDKԂ
		@return SDK̃C^tF[X
	 */
	virtual PHSdkIf* GetSdk()=0;

	/** @brief Solid쐬
		@param desc 쐬Solid̃fBXNv^
		@return Solid̃C^tF[X
	*/
	virtual PHSolidIf* CreateSolid(const PHSolidDesc& desc = PHSolidDesc())=0;
	
	/** @brief Solid̐擾
		@return Solid̐
	 */
	virtual int NSolids()const =0;

	/** @brief Solid擾
		@return Solid̃C^tF[X̔zւ̃|C^
	 */
	virtual PHSolidIf** GetSolids()=0;

	/** @brief w肵̓m̐ڐG̃[hݒ肷
		@param lhs g̕Е̍̂ւ̃|C^
		@param rhs ĝЕ̍̂ւ̃|C^
		@param mode ڐG̃[h

		lhsƍrhsƂ̐ڐG̃[hmodeɐݒ肷D
	  */
	virtual void SetContactMode(PHSolidIf* lhs,	PHSolidIf* rhs, PHSceneDesc::ContactMode mode = PHSceneDesc::MODE_LCP)=0;

	/** @brief w肵WɊ܂܂SĂ̍̓m̐ڐG̃[hݒ肷
		@param group ̂ւ̃|C^z̐擪AhX
		@param size |C^z̒
		@param mode ڐG̃[h

		group[0]group[size-1]܂ł̑SĂ̍̂̑g̐ڐG̃[hmodeɐݒ肷D
	 */
	virtual void SetContactMode(PHSolidIf** group, size_t length, PHSceneDesc::ContactMode mode = PHSceneDesc::MODE_LCP)=0;

	/** @brief w肵̂Ƒ̍̂Ƃ̑SĂ̐ڐG̃[hݒ肷
		@param mode ڐG̃[h
	 */
	virtual void SetContactMode(PHSolidIf* solid, PHSceneDesc::ContactMode mode = PHSceneDesc::MODE_LCP)=0;

	/** @brief V[ۗLSĂ̍̓m̐ڐG̃[hݒ肷
		@param mode ڐG̃[h
	 */
	virtual void SetContactMode(PHSceneDesc::ContactMode mode = PHSceneDesc::MODE_LCP)=0;

	///	@brief LCP\ǒvZ񐔂̎擾DMODE_LCP̏ꍇ̍S͂̌JԂߎvZ̉񐔁D
	virtual int GetNumIteration()=0;
	///	@brief LCP\ǒvZ񐔂̐ݒD
	virtual void SetNumIteration(int n)=0;
	
	/** @brief ֐߂쐬
		@param lhs ֐߂t鍄
		@param rhs ֐߂t鍄
		@param desc ֐߂̃fBXNv^
		@return ֐߂̃C^tF[X
		lhsƍrhsA֐߂쐬D
		descɂ͍쐬֐߂̎ނɑΉfBXNv^iPHJointDeschjnD
		lhsɃ\PbgCrhsɃvOtD
	 */
	virtual PHJointIf* CreateJoint(PHSolidIf* lhs, PHSolidIf* rhs, const IfInfo* ii, const PHJointDesc& desc)=0;
	template <class T> PHJointIf* CreateJoint(PHSolidIf* lhs, PHSolidIf* rhs, const T& desc){
		return CreateJoint(lhs, rhs, T::GetIfInfo(), desc);
	}
	
	/** @brief ֐߂̐擾
	 */
	virtual int NJoints()const=0;

	/** @brief ֐߂擾
	 */
	virtual PHJointIf* GetJoint(int i)=0;

	/** @brief [gm[h쐬
		@param root [gm[hƂȂ鍄
		@return [gm[h̃C^tF[X
		֐߃c[̍ƂȂm[h쐬D
	 */
	virtual PHRootNodeIf* CreateRootNode(PHSolidIf* root, const PHRootNodeDesc& desc = PHRootNodeDesc())=0;

	/** @brief [gm[h̐
	 */
	virtual int NRootNodes()const=0;

	/** @brief [gm[h擾
	 */
	virtual PHRootNodeIf* GetRootNode(int i)=0;

	/** @brief c[m[h쐬
		@param parent em[h̃C^tF[X
		@param child qm[hƂȂ鍄
		@return c[m[h̃C^tF[X
		childQƂc[m[h쐬C̃m[hparent̎qm[hƂĒǉD
		parentŵchildȂ֐߂CreateTreeNoděĂяoO
		CreateJointɂč쐬ĂȂ΂ȂȂD
		ɁCparent\PbgCchildvOłKvD
	 */
	virtual PHTreeNodeIf* CreateTreeNode(PHTreeNodeIf* parent, PHSolidIf* child, const PHTreeNodeDesc& desc = PHTreeNodeDesc())=0;

	/** @brief MA쐬
		@param lhs MAŘA֐
		@param rhs MAŘA֐
		@param desc MÃfBXNv^
		@return MÃC^tF[X
		1Rx֐lhsrhsAMA쐬D
		Aꂽ֐߂́C
		rhs->GetPosition() = desc.ratio * lhs->GetPosition()
		𖞂悤ɍSD
	 */
	virtual PHGearIf* CreateGear(PHJoint1DIf* lhs, PHJoint1DIf* rhs, const PHGearDesc& desc = PHGearDesc())=0;
	
	/** @brief MA̐
	 */
	virtual int NGears()const=0;

	/** @brief MA擾
	 */
	virtual PHGearIf* GetGear(int i)=0;

	/** @brief pX쐬
		@param desc pX̃fBXNv^
		pX쐬D
		pXPHPathJoint̋O\邽߂ɗpD
	 */
	virtual PHPathIf* CreatePath(const PHPathDesc& desc = PHPathDesc())=0;

	/** @brief ϕXebv擾
		@return ϕXebv
	 */
	virtual double GetTimeStep()const=0;
	
	/** @brief ϕXebvݒ肷
		@param dt ϕXebv
	 */
	virtual void SetTimeStep(double dt)=0;
	
	/** @brief JEg擾
		@return JEg
		V[쐬Ă猻݂܂łStep()s񐔂ԂD
	 */
	virtual unsigned GetCount()const=0;

	/** @brief JEgݒ肷
		@param count JEg
	 */
	virtual void SetCount(unsigned count)=0;
	
	/** @brief V[̎i߂
	 */
	virtual void Step()=0;
	virtual void ClearForce()=0;
	virtual void GenerateForce()=0;
	virtual void Integrate()=0;

	/** @brief V[ɂ
	 */
	virtual void Clear()=0;

	/** @brief d͂ݒ肷
		@param accel d͉xxNg
	 */
	//ɎׂvCNovodex͂ 
	virtual void SetGravity(const Vec3d& accel)=0;
	
	/** @brief d͂擾
		@return d͉xxNg
	 */
	virtual Vec3d GetGravity()=0;

	//virtual PHConstraintEngine* GetConstraintEngine()=0;
	//virtual PHGravityEngine*	GetGravityEngine()=0;
};

//@}

}	//	namespace Spr
#endif
