/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
/**
 *	@file SprPHSolid.h
 *	@brief 
*/
#ifndef SPR_PHSolidIf_H
#define SPR_PHSolidIf_H
#include <Foundation/SprObject.h>

/**	\addtogroup gpPhysics	*/
//@{
namespace Spr{;
///	̂̃Xe[g
struct PHSolidState{
	Vec3d		velocity;		///<	x			(Worldn)
	Vec3d		angVelocity;	///<	px			(Worldn)
	Posed		pose;			///<	ʒuƌ		(Worldn)
	Vec3d		force;			///<	O̐ϕł̍̂ɉ(Worldn)
	Vec3d		torque;			///<	O̐ϕł̍̂ɉgN(Worldn)
	Vec3d		nextForce;		///<	̐ϕł̍̂ɉ(Worldn)
	Vec3d		nextTorque;		///<	̐ϕł̍̂ɉgN(Worldn)
};
///	̂̃fBXNv^
struct PHSolidDesc: public PHSolidState{
	double		mass;			///<	
	Matrix3d	inertia;		///<	e\	(Localn)
	Vec3d		center;			///<	ʒS̈ʒu	(Localn)
	bool		dynamical;		///<	@ɏ]

	PHSolidDesc(){ Init(); }
	void Init(){
		mass = 1.0f;
		inertia = Matrix3d::Unit();
		dynamical = true;
	}
};

struct CDShapeIf; 
///	̂̃C^tF[X
struct PHSolidIf : public SceneObjectIf{
	IF_DEF(PHSolid);

	/** @brief ͂ʒSɉ
		@param f 
	 */
	virtual void AddForce(Vec3d f)=0;
	/** @brief gN
		@param t gN
	 */
	virtual void AddTorque(Vec3d t)=0;
	/**	@brief ͂w肵p_ɉ
		@param f 
		@param r V[Wnŕ\킵p_̈ʒu
	 */
	virtual void AddForce(Vec3d f, Vec3d r)=0;

	/**	@brief ʂ擾
		@return 
	 */
	virtual double		GetMass()=0;
	/** @brief ʂ̋t擾
		@return ʂ̋t
	 */
	virtual double		GetMassInv()=0;
	/** @brief ʂݒ肷
		@param m 
	 */
	virtual void		SetMass(double m)=0;
	/** @brief ʒS擾
		@return ʒS̈ʒu
	 */
	virtual Vec3d		GetCenterOfMass()=0;
	/** @brief ʒSݒ肷
		@param center ʒS̈ʒu
	 */
	virtual void		SetCenterOfMass(const Vec3d& center)=0;
	/** @brief e\擾
		@return e\
	 */
	virtual Matrix3d	GetInertia()=0;
	/** @brief e\̋t擾
		@return e\̋t
	 */
	virtual Matrix3d	GetInertiaInv()=0;
	/** @brief e\ݒ肷
		@param I e\
	 */
	virtual void		SetInertia(const Matrix3d& I)=0;
	/** @brief ̂̈ʒu擾
		@return V[ɑ΂鍄̃t[_̈ʒu
	 */
	virtual Vec3d		GetFramePosition() const =0;
	/** @brief ̂̈ʒuݒ肷
		@param p V[ɑ΂鍄̃t[_̈ʒu
	 */
	virtual void		SetFramePosition(const Vec3d& p) =0;
	/** @brief ̂̎ʒS̈ʒu擾
		@return V[ɑ΂鍄̂̎ʒS̈ʒu
	 */
	virtual Vec3d		GetCenterPosition() const  =0;
	/** @brief ̂̈ʒuݒ肷
		@param p V[ɑ΂鍄̂̎ʒS̈ʒu
	 */
	virtual void		SetCenterPosition(const Vec3d& p) =0;
	/** @brief ̂̑ÕXebv̕iړʂԂ
		@return ÕXebv̕iړ
	 */
	virtual Vec3d GetDeltaPosition(const Vec3d& p) const = 0;
	/** @brief ̂̑ÕXebv̕iړʂԂD
		@param pos dS̑ΈʒuD̈ʒu̕iړʂԂD
		@return ÕXebv̕iړ
	*/
	virtual Vec3d GetDeltaPosition() const = 0;
	/** @brief ̂̌擾
		@return V[ɑ΂鍄̂̌
	 */
	virtual Quaterniond GetOrientation() const =0;
	/** @brief ̂̌ݒ肷
		@param q V[ɑ΂鍄̂̌
	 */
	virtual void		SetOrientation(const Quaterniond& q) =0;

	/** @brief ̂̈ʒuƌ擾
		@return V[ɑ΂鍄̂̈ʒuƌ
	 */
	virtual Posed		GetPose() const = 0;
	/** @brief ̂̈ʒuƌݒ肷
		@param p V[ɑ΂鍄̂̈ʒuƌ
	 */
	virtual void		SetPose(const Posed& p) = 0;

	/** @brief ̂̑x擾
		@return V[ɑ΂鍄̂̎ʒS̑x
	 */
	virtual Vec3d		GetVelocity() const  =0;

	/** @brief ̂̑xݒ肷
		@param v V[ɑ΂鍄̂̎ʒS̑x
	 */
	virtual void		SetVelocity(const Vec3d& v) =0;
	/** @brief ̂̊px擾
		@return V[ɑ΂鍄̂̊px
	 */
	virtual Vec3d		GetAngularVelocity() const =0;
	
    /** @brief ̂̊pxݒ肷
		@param av V[ɑ΂鍄̂̊px
	 */
	virtual void		SetAngularVelocity(const Vec3d& av)=0;
    /** @brief Ō̃Xebvō̂ɉ͂擾B
		@return ̂ɉꂽ(Worldn)
	*/
	virtual Vec3d GetForce() const=0;
    /** @brief Ō̃Xebvō̂ɉgN擾B
		@return ̂ɉꂽgN(WorldnÂ̏dS)
	*/
	virtual Vec3d GetTorque() const=0;
	/** @brief ̂Ɍ`o^
		@param shape `ւ̃|C^
	 */
	virtual void		AddShape(CDShapeIf* shape)=0;
	/** @brief o^Ă`̌擾
		@return `̌
	 */
	virtual int			NShape()=0;
	/**	@brief o^Ă`擾
		@param index `̃CfbNX
		@return `ւ̃|C^
		ŏɓo^ꂽ`̃CfbNX0C̎ɓo^ꂽ`̃CfbNX1C
		ȉlD
	 */
	virtual CDShapeIf*	GetShape(int index)=0;
	/** @brief `̈ʒuƌ擾
		@param index ΏۂƂ`̃CfbNX
		@return ̂ɑ΂`̈ʒuƌ
		CfbNXɂĂGetShapeQƁD
	 */
	virtual Posed		GetShapePose(int index)=0;
	/** @brief `̈ʒuƌݒ肷
		@param index ΏۂƂ`̃CfbNX
		@param pose ̂ɑ΂`̈ʒuƌ
		CfbNXɂĂGetShapeQƁD
	 */
	virtual void		SetShapePose(int index, const Posed& pose)=0;
	
	/** @brief d͂L/
		@param bOn trueȂ΍̂ɏd͂DfalseȂΉȂD
	 */
	virtual void		SetGravity(bool bOn)=0;
	/** @brief @ɏ]ǂL/
		@param bOn trueȂ΍͕̂@ɂĉ^D
		@ɏ]Ȃ̂CSetPoseCSetVelocityȂǂ̊֐ɂʒu⑬x̍XV
		\łD
	 */
	virtual void		SetDynamical(bool bOn)=0;
	/** @brief @ɏ]ǂ擾
		@return trueȂ΍͕̂@ɂĉ^D
	 */
	virtual bool		IsDynamical()=0;
};

//@}

}	//	namespace Spr
#endif
