/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef AFFINE_H
#define AFFINE_H

/**	\addtogroup gpLinearAlgebra	*/
//@{
/** \page pageAffine @!JAAffinesNX̊Tv@!ENA Guide for Affine Matrix@!*
	\section secAffineIntro @!JA͂߂@!ENIntroduction@!*
		@!JA
		̃y[WAffinesNX(Spr::TAffine, Spr::TAffine2)̐łD
		AffinesNXCúC3DV~[VɕK{
		AffinesC++̃NXɂ̂łD
		@!EN
		This is a document for Affine Matrix Library.
		Affine Matrix Library is a set of classes for affine matrix,
		which are necessary for 3D simulation.
		@!*
	\section secAffineUsage @!JAg@!ENUsage@!*
	@!JA
	Affine sNXCúCwb_t@CȂ
	NXCuȂ̂, TAffine.h, TinyVec.h, TinyMat.h, TMatrix.h, TMatrixUtility.h, TVector.h
	𓯂tH_ɓĂC.cppt@Cwb_CN[h邾
	gpł܂D
	@!EN
	Affine Matrix Library consist only from some header files.
	You can use this library just coping some header files
	(TAffine.h, TinyVec.h, TinyMat.h, TMatrix.h, TMatrixUtility.h, TVector.h)
	and including them from your .cpp files.
	@!*
	\subsection secAffineSample @!JA Tv @!EN sample@!*
	\verbatim
#include "Affine.h"                         //  TAffines񃉃CũCN[hD
#include <iostream>

using namespace Spr;	//  @!JA AffinesNXSprOԂ̒Ő錾ĂD@!EN Affine matrix class is declared in the name space of Spr.@!*

void main(){
    Affinef af=Affinef::Rad(Rad(30), 'z');  //  @!JAvffloatTAffines錾. 
                                                @!ENDeclare an Affine matrix of float element.
                                                @!*
                                                @!JAz30x]sɏ
                                                @!ENInitializing as a matrix of rotation of 30 degrees around z axis.
                                                @!*
                                                
    Vec3f vec(1,0,0);                       //  @!JAvffloat3̃xNg錾
                                                @!ENDeclare a vector of float element.
                                                @!*
    std::cout << af;
    std::cout << vec << std::endl;
    std::cout << af * vec << std::endl;
}\endverbatim
	\subsection secAffineVecFunc @!JA AffinesExNg̃oƉZ  @!EN Functions of Affine matrixs and vectors. @!*
	@!JA
	ʂɉZł܂D
	<ul>
	<li> +:a, -:, *:/, /:萔1
	<li> ==:r, =:
	<li> <<:o, >>:
	<li> %:xNg̊O
	</ul>
	@!EN Normal calculations are supported.
	<ul>
	<li> +:addition, -:subtraction, *:multiplication/inner product, /:division
	<li> ==:comparison, =:assign
	<li> <<:output, >>:input
	<li> %:cross product
	</ul>
	@!*
	@!JA
	AffineΐC
	\verbatim
	TAffine<float> af; TVec3<float> v, af_v;
	af_v = af * v;\endverbatim
	Ƃ΂ł܂D
	@!EN
	Affine transformation is executed by:
	\verbatim
	TAffine<float> af; TVec3<float> v, af_v;
	af_v = af * v;\endverbatim
	@!*
	@!JA
	܂Ĉ悤TAffines̕oƂł܂D
	<ul>
		<li> af.Ex():	XxNgD(3xNg)D
		<li> af.Ey():	YxNgD(3xNg)D
		<li> af.Ez():	ZxNgD(3xNg)D
		<li> af.Trn():	sړD(3xNg)D
		<li> af.Rot():	]ϊD(3~Rs)D
	</ul>
	@!EN
	You can extract some portion of the Affine matrix as:
	<ul>
		<li> af.Ex():	base vector for x axis (3 dimensional vector)
		<li> af.Ey():	base vector for y axis (3 dimensional vector)
		<li> af.Ez():	base vector for z axis (3 dimensional vector)
		<li> af.Trn():	portion of translationD(3 dimensional vector)
		<li> af.Rot():	portion of rotationD(3x3 matrix)
	</ul>
	@!*
	@!JA ւ̑Ȃǂł܂D
	@!EN You can assign portion of affine matrx.
	@!*
	\verbatim
	TAffine<float> af;
	af.Pos() = Vec3f(10,0,0);
	af.Rot() = TAffine<float>::Rot(Rad(30), 'x').Rot() * af.Rot();\endverbatim
	@!JA xNĝ͎悤ȃo֐܂D
	@!EN a vector has following members.
	@!*
	<ul>
	<li> unit(): @!JA PʃxNgԂD
				 @!EN return a unit vector.
				 @!*
	<li> norm(): @!JA xNg̑傫(m)ԂD
				 @!EN return the size (norm) of a vector.
				 @!*
	</ul>
	\subsection secAffineConstruct Affines̏ 
	TAffines(Spr::TAffine)ɂ͕֗ȃRXgN^⏉֐pӂ܂D
	ŏЉ܂D
	<ul>
		<li> Spr::TAffine::Trn (T px, T py, T pz):	
				sړsɏDTTAffine<T>TDfloatdoubleȂǂŗǂD
		<li> Spr::TAffine::Rot(element_type th, char axis):
				]sԂDth̓WADaxiśC'x', 'y', 'z'Delement_typeT̂ƁD
		<li> Spr::TAffine::ProjectionD3D (TVec3 screen, TVec2 size, T front=1.0f, T back=10000.0f):
				D3DpˉesƂďD
		<li> Spr::TAffine::ProjectionGL (TVec3 screen, TVec2 size, T front=1.0f, T back=10000.0f):
		<br>
			OpenGLpˉesƂď(-ZO)D
			<ul>
				<li> screen  J猩r[|[g̒S̈ʒu
				<li> size    r[|[g̃TCY
				<li> front   ÕNbsOʂƃJ̋
				<li> back    ̃NbsOʂƃJ̋
			</ul>
		<li> Spr::TAffine::LookAtGL (TVec3 posz, TVec3 posy)
			ʒu͂̂܂܂ŁCposz-Ez(), posy  Ey()悤Affines
	</ul>
	
	\section thanks ӎ
	LUCtsCKEX@Ȃǂ̍svZASÝC<br>
    uwbɂŐVASYTxS\[XR[hv<br>
    ftp://ftp.matsusaka-u.ac.jp/pub/algorithms<br>
	 F Haruhiko Okumura<br>
	ςėpĂ܂D
	RɃR[hg킹ĂāC肪Ƃ܂D
	
*/

#include "TMatrix.h"
#include "TMatrixUtility.h"
#include "TinyVec.h"
#include "TinyMat.h"
#include "VectorDebug.h"

/**	@file Affine.h 2/3AtBs*/

#ifndef PTM_PACK	//	P̂Ŏgpꍇ́Cnamespace ɓȂ
namespace Spr {
#endif

#ifdef _WIN32
 #ifdef _DEBUG
  #pragma optimize ("awgity", on)
  #pragma auto_inline(on)
  #pragma inline_recursion(on)
 #endif
#endif

#undef M_PI
#ifdef __BORLANDC__
#define M_PI 3.14159265358979323846
#else
///	~
const double M_PI = 3.14159265358979323846;
#endif

#undef abs
#ifdef __BORLANDC__
/*	ΒlDBCB6Ⴄ֐NĂ܂(ԂoO)̂ŁCtemplate ͎gpĂȂ
	std::abs ƂԂ邽߂ƎvD	*/
#define DEF_ABS_FUNC(T)		inline T abs(T t){ return t > T()  ?  t  :  -t; }
DEF_ABS_FUNC(float)
DEF_ABS_FUNC(double)
DEF_ABS_FUNC(char)
DEF_ABS_FUNC(int)
DEF_ABS_FUNC(long)
#else
template <class T> T abs(T t){ return t > T()  ?  t  :  -t; }
#endif

#undef sign
///	(Ȃ1, Ȃ -1 Ԃ)
template <class T> T sign(T t){
	return t > T()  ?  T(1)  :  T(-1);
}

#undef min
/// Ԃ
template <class T> T min(T a, T b){return a < b ? a : b;}

#undef max
/// 傫Ԃ
template <class T> T max(T a, T b){return a > b ? a : b;}

/// ςƂ
template <class T> T ave(T a, T b){return T(0.5 * (a + b));}

///	xWAɕϊ
inline double Rad(double deg){
	return ((double)deg/360*2*M_PI);
}
inline float Radf(double deg){
	return (float)((double)deg/360*2*M_PI);
}
///	WAxɕϊ
inline double Deg(double rad){
	return (rad/(2*M_PI)) * 360;
}
inline float Degf(double rad){
	return (float)(rad/(2*M_PI)) * 360;
}
///	2
template <class SC>
inline SC Square(SC x){
	return x*x;
}
///	2x2s̍s
template <class SC>
inline SC Det2(SC a, SC b, SC c, SC d){
	return ((a)*(d) - (b)*(c));
}

//-----------------------------------------------------------------------------
//	TAffine2

/**	TAffine2s(],g,sړ\)s.
	Tv́C\ref pageAffine QƁD
*/
template <class T>
class TAffine2:public PTM::TMatrixBase<DIMENC(3),DIMENC(3),
	PTM::TMatrixDescCol< TAffine2<T>, PTM::TMatrixRow<3,3,T>, 3,3,3,T> >{
public:
	typedef PTM::TMatrixDescCol< TAffine2<T>, PTM::TMatrixRow<3,3,T>, 3,3,3,T> desc;
	typedef PTM::TMatrixBase<DIMENC(3),DIMENC(3),desc> base_type;
	///	{Iȃo̒` @see ::DEF_MATRIX_BASIC_MEMBER
	DEF_MATRIX_BASIC_MEMBER(TAffine2);
	union{
		struct{
			T xx, xy, xz;
			T yx, yy, yz;
			T px, py, pz;
		};
		T data[3][3];
	};
	///	vf̃ANZX
	element_type& item_impl(size_t i, size_t j){ return data[j][i]; }
	const element_type& item_impl(size_t i, size_t j) const { return data[j][i]; }
	
	/**@name	xNgւ̃ANZX	*/
	//@{
	/// 
	TVec2<element_type>& Ex(){
		return *(TVec2<element_type>*) &this->item(0,0);
	}
	/// 
	const TVec2<element_type>& Ex() const{
		return *(TVec2<element_type>*) &this->item(0,0);
	}
	/// 
	TVec2<element_type>& Ey(){
		return *(TVec2<element_type>*) &this->item(0,1);
	}
	/// 
	const TVec2<element_type>& Ey() const{
		return *(TVec2<element_type>*) &this->item(0,1);
	}
	/// 
	TVec2<element_type>& Trn(){
		return *(TVec2<element_type>*) &this->item(0,2);
	}
	/// 
	const TVec2<element_type>& Trn() const{
		return *(TVec2<element_type>*) &this->item(0,2);
	}
	/// 
	TVec2<element_type>& Pos(){ return Trn(); }
	/// 
	const TVec2<element_type>& Pos() const { return Trn(); }
	//@}

	/**@name	vfւ̃ANZX	*/
	//@{
	/// 
	element_type& ExX() {return Ex().X();}
	/// 
	element_type& ExY() {return Ex().Y();}
	/// 
	element_type& EyX() {return Ey().X();}
	/// 
	element_type& EyY() {return Ey().Y();}
	/// 
	element_type& TrnX() {return Trn().X();}
	/// 
	element_type& TrnY() {return Trn().Y();}
	///	TrnX()̕ʖ
	element_type& PosX() {return Trn().X();}
	///	TrnY()̕ʖ
	element_type& PosY() {return Trn().Y();}
	//@}

	///	]gϊo.
	PTM::TSubMatrixCol<2,2, desc>& Rot() { return sub_matrix(0,0,PTM::TSubMatrixCol<2,2, desc>()); }
	///	]gϊo (const).
	const PTM::TSubMatrixCol<2,2, desc>& Rot() const { return sub_matrix(0,0,PTM::TSubMatrixCol<2,2, desc>()); }
	
	/**@name	ƍ\z	*/
	//@{
	///	Pʍs
	static TAffine2<T> Unit(){
		TAffine2<T> y;
		PTM::init_unitize(y);
		return y;
	}
	///	sړ
	static TAffine2<T> Trn(element_type px, element_type py){
		TAffine2<T> y;
		y.Trn().X() = px;
		y.Trn().Y() = py;
		return y;
	}
	///	]CȂЂƂVC.netŃG[ɂȂD
	static TAffine2<T> Rot(element_type th, int d=0){
		TAffine2 y;
		PTM::init_rot(y.Rot(), th);
		return y;
	}
	/// g
	static TAffine2<T> Scale(element_type sx, element_type sy){
		TAffine2<T> y;
		y.item(0, 0) = sx; y.item(1, 1) = sy;
		return y;
	}
	///RXgN^
	void set_default(){PTM::init_unitize(*this);}
	//@}
};

//-----------------------------------------------------------------------------
//	TAffine
/**	TAffines(],g,sړ\)s.
	Tv́C\ref pageAffine QƁD	*/

template <class T>
class TAffine:public PTM::TMatrixBase<DIMENC(4),DIMENC(4),
	PTM::TMatrixDescCol< TAffine<T>, PTM::TMatrixRow<4,4,T>, 4,4,4,T> >{
public:
	typedef PTM::TMatrixDescCol< TAffine<T>, PTM::TMatrixRow<4,4,T>, 4,4,4,T> desc;
	typedef PTM::TMatrixBase<DIMENC(4),DIMENC(4),desc> base_type;
	/**	pȂ{Iȃo̒`.
		@see ::DEF_MATRIX_BASIC_MEMBER	*/
	DEF_MATRIX_BASIC_MEMBER(TAffine);
	union{
		struct{
			T xx, xy, xz, xw;
			T yx, yy, yz, yw;
			T zx, zy, zz, zw;
			T px, py, pz, pw;
		};
		T data[4][4];
	};
	///	vf̃ANZX
	element_type& item_impl(size_t i, size_t j){ return data[j][i]; }
	const element_type& item_impl(size_t i, size_t j) const { return data[j][i]; }

	/**@name	xNgւ̃ANZX	*/
	//@{
	/// 
	TVec3<element_type>& Ex() { return (TVec3<element_type>&)this->col(0); }
	/// 
	const TVec3<element_type>& Ex() const { return (TVec3<element_type>&)this->col(0); }
	/// 
	TVec3<element_type>& Ey() { return (TVec3<element_type>&)this->col(1); }
	/// 
	const TVec3<element_type>& Ey() const { return (TVec3<element_type>&)this->col(1); }
	/// 
	TVec3<element_type>& Ez() { return (TVec3<element_type>&)this->col(2); }
	/// 
	const TVec3<element_type>& Ez() const { return (TVec3<element_type>&)this->col(2); }
	/// 
	TVec3<element_type>& Trn() { return (TVec3<element_type>&)this->col(3); }
	/// 
	const TVec3<element_type>& Trn() const { return (TVec3<element_type>&)this->col(3); }
	///	sړ(Trn()̕ʖ)
	TVec3<element_type>& Pos() {return Trn();}
	///	sړ(Trn()̕ʖ,const )
	const TVec3<element_type>& Pos() const {return Trn();}
	//@}

	/**@name	vfւ̃ANZX	*/
	//@{
	/// 
	element_type& ExX() {return Ex().X();}
	const element_type& ExX() const {return Ex().X();}
	/// 
	element_type& ExY() {return Ex().Y();}
	const element_type& ExY() const {return Ex().Y();}
	/// 
	element_type& ExZ() {return Ex().Z();}
	const element_type& ExZ() const {return Ex().Z();}
	/// 
	element_type& EyX() {return Ey().X();}
	const element_type& EyX() const {return Ey().X();}
	/// 
	element_type& EyY() {return Ey().Y();}
	const element_type& EyY() const {return Ey().Y();}
	/// 
	element_type& EyZ() {return Ey().Z();}
	const element_type& EyZ() const {return Ey().Z();}
	/// 
	element_type& EzX() {return Ez().X();}
	const element_type& EzX() const {return Ez().X();}
	/// 
	element_type& EzY() {return Ez().Y();}
	const element_type& EzY() const {return Ez().Y();}
	/// 
	element_type& EzZ() {return Ez().Z();}
	const element_type& EzZ() const {return Ez().Z();}
	/// 
	element_type& TrnX() {return Trn().X();}
	const element_type& TrnX() const {return Trn().X();}
	/// 
	element_type& TrnY() {return Trn().Y();}
	const element_type& TrnY() const {return Trn().Y();}
	/// 
	element_type& TrnZ() {return Trn().Z();}
	const element_type& TrnZ() const {return Trn().Z();}
	///	TrnX()̕ʖ
	element_type& PosX() {return TrnX();}
	const element_type& PosX() const {return TrnX();}
	///	TrnY()̕ʖ
	element_type& PosY() {return TrnY();}
	const element_type& PosY() const {return TrnY();}
	///	TrnZ()̕ʖ
	element_type& PosZ() {return TrnZ();}
	const element_type& PosZ() const {return TrnZ();}
	///
	element_type& ExW() {return this->item(3,0);}
	const element_type& ExW() const {return this->item(3,0);}
	///
	element_type& EyW() {return this->item(3,1);}
	const element_type& EyW() const {return this->item(3,1);}
	///
	element_type& EzW() {return this->item(3,2);}
	const element_type& EzW() const {return this->item(3,2);}
	///
	element_type& TrnW() {return this->item(3,3);}
	const element_type& TrnW() const {return this->item(3,3);}
	///
	element_type& PosW() {return this->item(3,3);}
	const element_type& PosW() const {return this->item(3,3);}
	//@}

	///	]gϊւ̎QƂԂ.
	PTM::TSubMatrixCol<3,3, desc>& Rot(){ return this->sub_matrix(PTM::TSubMatrixDim<0,0,3,3>()); }
	///	]gϊւ̎QƂԂ (const).
	const PTM::TSubMatrixCol<3,3, desc>& Rot() const { return this->sub_matrix(PTM::TSubMatrixDim<0,0,3,3>()); }

	/**@name	ƍ\z	*/
	///	Pʍs
	static TAffine<T> Unit(){
		TAffine<T> y;
		PTM::init_unitize(y);
		return y;
	}
	///	sړ
	static TAffine<T> Trn(element_type px, element_type py, element_type pz){
		TAffine<T> y;
		y.Trn().X() = px;
		y.Trn().Y() = py;
		y.Trn().Z() = pz;
		return y;
	}
	///	x/y/z܂]
	static TAffine<T> Rot(element_type th, char axis)
	{
		TAffine<T> y;
#ifdef __BORLANDC__
        TMatrix3<T> r = y.Rot();
		PTM::init_rot(r, th, axis);
#else
		PTM::init_rot(y.Rot(), th, axis);
#endif
		return y;
	}
	/**	Cӎ܂]
\verbatim
		+																	   +
		|u^2+(1-u^2)cos(th)      uv(1-cos(th))-wsin(th)  wu(1-cos(th))+vsin(th)|
	R =	|uv(1-cos(th))+wsin(th)  v^2+(1-v^2)cos(th)      vw(1-cos(th))-usin(th)|
		|wu(1-cos(th))-vsin(th)  vw(1-cos(th))+usin(th)  w^2+(1-w^2)cos(th)    |
		+																	   +
\endverbatim
*/
	template <class BUF>
	static TAffine<T> Rot(element_type th, const PTM::TVectorBase<DIMENC(3), BUF>& axis)
	{
		TAffine<T> y;
		Matrix3f r;
		PTM::init_rot(r, th, axis);
		y.Rot() = r;
		return y;
	}
	/// g
	static TAffine<T> Scale(element_type sx, element_type sy, element_type sz){
		TAffine<T> y;
		y.item(0, 0) = sx; y.item(1, 1) = sy; y.item(2, 2) = sz;
		return y;
	}
	/**	OpenGL̎ˉesƂď
		@param screen	J猩r[|[g̒S̈ʒu
		@param size		r[|[g̃TCY
		@param front	ÕNbsOʂƃJ̋
		@param back		̃NbsOʂƃJ̋	*/
	template <class BUFS, class BUFZ>
	static TAffine<T> ProjectionGL(
		const PTM::TVectorBase<DIMENC(3), BUFS>& screen,
		const PTM::TVectorBase<DIMENC(2), BUFZ>& size,
		element_type front=1.0f, element_type back=10000.0f)
	{
		TAffine<T> y;
		PTM::init_projection_gl(y, screen, size, front, back);
		return y;
	}
	/**	Direct3D̎ˉesƂď
		@param screen	J猩r[|[g̒S̈ʒu
		@param size		r[|[g̃TCY
		@param front	ÕNbsOʂƃJ̋
		@param back		̃NbsOʂƃJ̋	*/
	template <class BUFS, class BUFZ>
	static TAffine<T> ProjectionD3D(const PTM::TVectorBase<DIMENC(3), BUFS>& screen,
	const PTM::TVectorBase<DIMENC(2), BUFZ>& size,
		element_type front=1.0f, element_type back=10000.0f)
	{
		TAffine<T> y;
		PTM::init_projection_d3d(y, screen, size, front, back);
		return y;
	}

	///	posiEz()TAffines
	template <class BUF>
	void LookAt(const PTM::TVectorBase<DIMENC(3), BUF>& posi)
	{
		PTM::init_look_at(*this, posi);
	}
	///	poszEz(), posy  Ey() TAffines
	template <class BUFZ, class BUFY>
	void LookAt(const PTM::TVectorBase<DIMENC(3), BUFZ>& posz, const PTM::TVectorBase<DIMENC(3), BUFY>& posy)
	{
		PTM::init_look_at(*this, posz, posy);
	}
	///	posi-Ez()TAffines
	template <class BUF>
	void LookAtGL(const PTM::TVectorBase<DIMENC(3), BUF>& posi)
	{
		PTM::init_look_at_gl(*this, posi);
	}
	///	posz-Ez(), posy  Ey() TAffines
	template <class BUFZ, class BUFY>
	void LookAtGL(const PTM::TVectorBase<DIMENC(3), BUFZ>& posz, const PTM::TVectorBase<DIMENC(3), BUFY>& posy)
	{
		PTM::init_look_at_gl(*this, posz, posy);
	}
	
	///RXgN^
	void set_default(){PTM::init_unitize(*this);}
};

/**	TAffinẽRXgN^`̃}NDRXgN^͌pȂC
	Define邱ƂŁCRXgN^pƂłD	*/
#define DEF_TAFFINE_CONSTRUCTORS(TAffine)												\
	TAffine(){*this=Unit();}															\
	TAffine(element_type px, element_type py, element_type pz){*this=Trn(px, py, pz);}	\
	template <class BUFX, class BUFY>													\
	TAffine(const PTM::TVectorBase<DIMENC(3), BUFX>& exi,								\
			const PTM::TVectorBase<DIMENC(3), BUFY>& eyi){								\
			PTM::init_direct(Rot(), exi, eyi, 'x');										\
			item(3, 0) = 0; item(3, 1) = 0; item(3, 2) = 0; item(3, 3) = 1;				\
			item(0, 3) = 0; item(1, 3) = 0; item(2, 3) = 0;								\
	}																					\
	template <class BUFX, class BUFY, class BUFP>										\
	TAffine(	const PTM::TVectorBase<DIMENC(3), BUFX>& exi,							\
			const PTM::TVectorBase<DIMENC(3), BUFY>& eyi,								\
			const PTM::TVectorBase<DIMENC(3), BUFP>& posi){								\
			PTM::init_direct(Rot(), exi, eyi, 'x');										\
			item(3, 0) = 0; item(3, 1) = 0; item(3, 2) = 0; item(3, 3) = 1;				\
			item(0, 3) = posi.X(); item(1, 3) = posi.Y(); item(2, 3) = posi.Z();		\
	}                                                                                   \
	template <class BUFA, class BUFB>													\
	TAffine(	const PTM::TVectorBase<DIMENC(3), BUFA>& a,								\
			const PTM::TVectorBase<DIMENC(3), BUFB>& b,									\
			char axis){																	\
			PTM::init_direct(Rot(), exi, eyi, axis);									\
			item(3, 0) = 0; item(3, 1) = 0; item(3, 2) = 0; item(3, 3) = 1;				\
			item(0, 3) = 0; item(1, 3) = 0; item(2, 3) = 0;								\
	}																					\
	template <class BUFA, class BUFB, class BUFP>										\
	TAffine(	const PTM::TVectorBase<DIMENC(3), BUFA>& a,							    \
			const PTM::TVectorBase<DIMENC(3), BUFB>& b,									\
			char axis, const PTM::TVectorBase<DIMENC(3), BUFP>& posi){					\
			PTM::init_direct(Rot(), exi, eyi, axis);									\
			item(3, 0) = 0; item(3, 1) = 0; item(3, 2) = 0; item(3, 3) = 1;				\
			item(0, 3) = posi.X(); item(1, 3) = posi.Y(); item(2, 3) = posi.Z();		\
			}

/**	TAffine2̃RXgN^`̃}NDRXgN^͌pȂC
	Define邱ƂŁCRXgN^pƂłD	*/
#define DEF_TAFFINE_CONSTRUCTORS2(TAffine)												\
	TAffine(element_type th, char axis) {												\
		*this=Rot(th, axis);															\
	}																					\
	template <class BUF>																\
	TAffine(element_type th, char axis,													\
		const PTM::TVectorBase<DIMENC(3), BUF>& posi) {									\
		*this = Rot(th, axis); Pos() = posi; }											\
	template <class BUFA>																\
	TAffine(element_type th, const PTM::TVectorBase<DIMENC(3), BUFA>& axis){			\
		*this = Rot(th, axis.unit());			                                        \
	}                                                                                   \
	template <class BUFA, class BUFP>													\
	TAffine(element_type th, const PTM::TVectorBase<DIMENC(3), BUFA>& axis , const PTM::TVectorBase<DIMENC(3), BUFP>& posi){	\
		*this = Rot(th, axis.unit()); Pos() = posi;										\
	}																					\
	template <class BUFS, class BUFZ>													\
	TAffine(const PTM::TVectorBase<DIMENC(3), BUFS>& screen, const PTM::TVectorBase<DIMENC(2), BUFZ>& size, element_type front=1.0f, element_type back=10000.0f){	\
		*this = ProjectionGL(screen, size, front, back);								\
	}                                                                                   \
	template <class BUF, class BUFV>													\
	TAffine(const PTM::TMatrixOp<3, 3, BUF>& m, const PTM::TVectorBase<DIMENC(3), BUFV> posi){\
		Rot() = m; Pos() = posi; ExW() = 0; EyW() = 0; EzW() = 0; PosW() = 1;			\
	}


#ifdef _WIN32
 #pragma warning (disable: 4700)
#endif
///	TAffineƃxNg̊|Z
template <class T, class TV>
TVec3<TV> operator * (
	const TAffine<T>& a,
	const TVec3<TV>& b){
	TVec3<TV> r;
	r[0] = a[0][0]*b[0] + a[0][1]*b[1] + a[0][2]*b[2] + a[0][3];
	r[1] = a[1][0]*b[0] + a[1][1]*b[1] + a[1][2]*b[2] + a[1][3];
	r[2] = a[2][0]*b[0] + a[2][1]*b[1] + a[2][2]*b[2] + a[2][3];
	return r;
}
#ifdef _WIN32
 #pragma warning (default: 4700)
#endif

///	float2AtBs.
typedef TAffine2<float> Affine2f;
///	double2AtBs.
typedef TAffine2<double> Affine2d;
///	float3AtBs.
typedef TAffine<float> Affinef;
///	double3AtBs.
typedef TAffine<double> Affined;
//@}


#ifdef _WIN32
 #ifdef _DEBUG
  #pragma optimize ("", on)
  #pragma auto_inline(off)
  #pragma inline_recursion(off)
 #endif
#endif

#ifndef PTM_PACK	//	P̂Ŏgpꍇ́Cnamespace ɓȂ
}	//	namespace Spr
#endif

#endif
