/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef CDDETECTORIMP_H
#define CDDETECTORIMP_H
#include "CDQuickHull3D.h"
#include "CDConvexMesh.h"
#include "CDSphere.h"
#include "CDBox.h"

namespace Spr {;

#define CD_EPSILON	HULL_EPSILON
#define CD_INFINITE	HULL_INFINITE

class CDFace;
class CDConvex;

/// Shapȇg̏
struct CDShapePairState{
	Vec3d normal;				///<	Փ˂̖@(01) (Global)
	double depth;				///<	Փ˂̐[FŋߖT_߂邽߂ɁC2̂𓮂D
	unsigned lastContactCount;	///<	ŌɐڐG
};
class CDShapePair: public CDShapePairState, public UTRefCount{
public:
	CDConvex* shape[2];			// Ώۂ̌`
	Posed shapePoseW[2];		// Worldnł̌`̎p̃LbV

	//	ڐG茋
	Vec3d	closestPoint[2];		///< ŋߖT_i[JWnj
	Vec3d	commonPoint;			///< ̓_iWorldWnj
	Vec3d	center;					///< 2̍ŐN_̒Ԃ̓_
	enum State{
		NONE,		//	ڐGȂ
		NEW,		//	OڐGȂōڐG
		CONTINUE,	//	OڐG
	} state;
	Vec3d iNormal;				///<	ϕɂ@

public:
	CDShapePair(){
	}
	void SetState(const CDShapePairState& s){
		(CDShapePairState&)*this = s;
	}	
	///	ڐG
	bool Detect(unsigned ct, CDConvex* s0, CDConvex* s1, const Posed& pose0, const Posed& pose1);
	///	AڐGDɖ@/SȂǂvZĂ܂D
	bool DetectContinuously(unsigned ct, CDConvex* s0, CDConvex* s1, const Posed& pose0, const Vec3d& delta0, const Posed& pose1, const Vec3d& delta1);
	///	@̌vZ
	void CalcNormal();
};

///	BBoxm̌DĂ trueD
bool FASTCALL BBoxIntersection(Posed postureA, Vec3f centerA, Vec3f extentA,
					  Posed postureB, Vec3f centerB, Vec3f extentB);

///	GJKŋL_
bool FASTCALL FindCommonPoint(const CDConvex* a, const CDConvex* b,
					 const Posed& a2w, const Posed& b2w,
					 Vec3d& v, Vec3d& pa, Vec3d& pb);
///	GJKŋL_DA, range͐K
int FASTCALL ContFindCommonPoint(const CDConvex* a, const CDConvex* b,
	const Posed& a2w, const Posed& b2w, Vec3d& range, Vec3d& normal, Vec3d& pa, Vec3d& pb, double& dist);

#if 1
/// GJKōŋߖT_΂
void FASTCALL FindClosestPoints(const CDConvex* a, const CDConvex* b,
					   const Posed& a2w, const Posed& b2w,
					   Vec3d& pa, Vec3d& pb);
#endif

class CDFace;


///	ContactAnalysisgpʑʑ̖̂ʂ\D
class CDContactAnalysisFace{
public:
	class DualPlanes: public std::vector<CDQHPlane<CDContactAnalysisFace>*>{};

	CDFace* face;	///<	ʂoΕϊ_QuickHull̂ŁCCDFaceVtxs.
	int id;			///<	ǂSolid̖ʂ\IDD

	//@group CDContactAnalysis ̌`߂ۂɎgƗ̈
	//@{
	Vec3f normal;	///<	ʂ̖@xNg
	float dist;		///<	_̋
	/**	QuickHullASYp[NGAD
		ԉ_猩ʂ폜Ǝc`̃GbW
		ڑĂ߂̃|C^D
		_ʂ̐ڑD	ʁ_͒_̕я番D	*/
	CDQHPlane<CDContactAnalysisFace>* horizon;
	//@}
	
	///	QuickHullɂƂĂ̒_D̖ʂoΕϊĂł钸_
	Vec3f GetPos() const { return normal / dist; }
	/**	oΕϊsDbaseɓn_obt@́CoΕϊ\Ȃ悤
		oΕϊ̒S_ƂȂ悤ȍWnłȂ΂ȂȂD	*/
	bool CalcDualVtx(Vec3f* base);

	/**	̖ʂoΕϊĂł钸_܂ޖʁD
		܂Č`\钸_̂C
		̖ʂɂ	*/
	DualPlanes dualPlanes;
	///	̌`\钸_̖̂ʂɂ̂̐.
	size_t NCommonVtx(){ return dualPlanes.size(); }
	///	̌`\钸_̖̂ʂɂ.
	Vec3f CommonVtx(int i);
	///	fobOp\
	void Print(std::ostream& os) const;
};
inline std::ostream& operator << (std::ostream& os, const CDContactAnalysisFace& f){
	f.Print(os);
	return os;
}

/**	̉͂NXD(̌`߂/̖@ϕŋ߂)	*/
class CDContactAnalysis{
public:
	/// \defgroup quickHull QuickHull̂߂̒_ƕ
	//@{
	
	typedef std::vector<CDContactAnalysisFace*> Vtxs;	
	static Vtxs vtxs;					///<	QuickHull̒_
	typedef std::vector<CDContactAnalysisFace> VtxBuffer;	
	static VtxBuffer vtxBuffer;

	static CDQHPlanes<CDContactAnalysisFace> planes;	///<	
	bool isValid;						///<	̃|S͗LH
	//@}
	std::vector<Vec3f> tvtxs[2];		///<	Ώۂ2̓ʑʑ̂CommonPointnł̒_̍W
	/**	ʕ̌`߂D
		ʂ́Cʕ\ʂ vtxs.begin() Ԃl܂łɁC
		ʕ\钸_C planes.begin  planes.end ̂
		deleted==false ̂̂ɓĕԂD
		cp  shapePoseW  shape[0], shape[1]̒_Worldnɕϊ
		ϊs񂪓ĂȂ΂ȂȂD	*/
	CDContactAnalysisFace** FindIntersection(CDShapePair* cp);
	/**	̌`̖@ϕāCՓ˂̖@߂D
		AB̏Փ˂̖@́C̖ʂ̂CA̖ʂ̖@̐ϕ
		B̖ʂ̖@̐ϕ̂ɂȂD	*/
	void IntegrateNormal(CDShapePair* cp);
	/**	@̌vZDO̖@̌ɕ̂𓮂C
		̂𗣂čŋߖT_߂D	*/
	void CalcNormal(CDShapePair* cp);
};

}
#endif
