/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef CDQUICKHULL2DIMP_H
#define CDQUICKHULL2DIMP_H

#include "CDQuickHull2D.h"
#include <Base/Affine.h>
#include <Base/BaseDebug.h>
#include <vector>
#include <float.h>

namespace Spr{;

template <class TVtx>
void CDQHLine<TVtx>::Clear(){			///<	NADgOɌĂԁD
	deleted = false;
}
template <class TVtx>
bool CDQHLine<TVtx>::Visible(TVtx* p){
	Vec2d pos = p->GetPos();
	return pos*normal > dist;
}
template <class TVtx>
int CDQHLine<TVtx>::GetVtxID(TVtx* v){
	int i;
	for(i=0; i<2; ++i) if(v == vtx[i]) break;
	return i;
}
template <class TVtx>
void CDQHLine<TVtx>::CalcNormal(){
	Vec2d pos0 = vtx[0]->GetPos();
	Vec2d pos1 = vtx[1]->GetPos();
	Vec2d a = pos1 - pos0;
	normal = Vec2d(-a.y, a.x);
	assert(normal.norm());
	normal.unitize();
	dist = pos0 * normal;
}
template <class TVtx>
double CDQHLine<TVtx>::CalcDist(TVtx* v){
	Vec2d vPos = v->GetPos();
	Vec2d diffMin = vPos - Vec2d(vtx[0]->GetPos());
	return diffMin * normal;
}
template <class TVtx>
void CDQHLine<TVtx>::Print(std::ostream& os) const {
	os << "Plane:";
	for(int i=0; i<2; ++i){
		os << " " << *vtx[i];
	}
	os << " d:" << dist << " n:" << normal;
	if (deleted) os << " del";
	os << std::endl;
}

template <class TVtx>
std::ostream& operator << (std::ostream& os, const CDQHLine<TVtx>& pl){
	pl.Print(os);
	return os;
}

template <class TVtx>
unsigned CDQHLines<TVtx>::size(){
	return end - begin; 
}
template <class TVtx>
CDQHLines<TVtx>::CDQHLines(int l):epsilon(1e-6), infinite(1e8), len(l){
	buffer = new CDQHLine[len];
	Clear();
}
template <class TVtx>
void CDQHLines<TVtx>::Clear(){
	begin = buffer;
	end = begin;
	nLines = 0;
}
template <class TVtx>
void CDQHLine<TVtx>::Reverse(){	
	normal = -normal;
	dist = -dist;
	std::swap(vtx[0], vtx[1]);
}
template <class TVtx>
CDQHLines<TVtx>::~CDQHLines(){
	delete [] buffer;
}
/**	be܂ł̒_ʕDgp_bvtxBeginC
gpȂ_́CvtxBegineɈړD	
beginend͒_3܂ޖʂɂȂD̖ʂʕɎgꂽ
 CDQHLine::deleted  false ɂȂĂD	*/
template <class TVtx>
void CDQHLines<TVtx>::CreateConvexHull(TVtx** b, TVtx** e){
	if (e-b < 2){
		if (e-b == 1){
			end->Clear();
			end->vtx[0] = b[0];
			end->vtx[1] = b[0];
			end->neighbor[0] = end->neighbor[1] = end;
			end++;
		}
		return;
	}
	vtxBeginInput = b;
	vtxEndInput = e;
	vtxBegin = b;
	vtxEnd = e;
	//	ŏ̖ʂ
	if (CreateFirstConvex()){
		HULL_DEBUG_EVAL(
			DSTR << "First:" << begin->vtx[0]->GetPos() << begin->vtx[1]->GetPos() << std::endl;
			if ((begin->vtx[1]->GetPos() - begin->vtx[0]->GetPos()).norm() < 0.001) {
				DSTR << "same error" << std::endl;
			}
			DSTR << begin->dist << begin->normal << std::endl;
		)
		for(CDQHLine* cur = begin; cur != end; ++cur){
			if (cur->deleted) continue;
			TreatPlane(cur);
			assert(end < buffer+len);
		}
	}
}
	
template <class TVtx>
void CDQHLines<TVtx>::Print(std::ostream& os) const {
	int num=0;
	for(const CDQHLine* it = begin; it != end; ++it){
		if (!it->deleted) num ++;
	}
	os << num << " lines." << std::endl;
	for(const CDQHLine* it = begin; it != end; ++it){
		it->Print(os);
	}
}

template <class TVtx>
bool CDQHLines<TVtx>::CreateFirstConvex(){
	CDQHLines& lines = *this;
	double minVal, maxVal;
	//typename TVtxs::iterator it, minVtx, maxVtx;
	TVtx** it, **minVtx, **maxVtx;	
	minVal = maxVal = (*vtxBegin)->GetPos().X();
	minVtx = maxVtx = vtxBegin;
	//	x̍őƍŏ
	for(it = vtxBegin+1; it != vtxEnd; ++it){
		double x = (*it)->GetPos().X();
		if (x < minVal){
			minVal = x;
			minVtx = it;
		}
		if (x > maxVal){
			maxVal = x;
			maxVtx = it;
		}
	}
	if (maxVal - minVal < epsilon){	//	 x̍őEŏ
		//	y̍őŏ
		for(it = vtxBegin+1; it != vtxEnd; ++it){
			double x = (*it)->GetPos().Y();
			if (x < minVal){
				minVal = x;
				minVtx = it;
			}
			if (x > maxVal){
				maxVal = x;
				maxVtx = it;
			}
		}
	}
	if (maxVal - minVal < epsilon){	//	ǂ1_Ȃ
		return false;
	}

	//	őŏCŏŏ2Ԗڂɒu
	std::swap(*maxVtx, vtxBegin[0]);		//	擪ƍőւ
	if (minVtx == vtxBegin){				//	擪ŏ
		std::swap(*maxVtx, vtxBegin[1]);	//	ő傾ꏊ=擪Ăꏊŏ
	}else{
		std::swap(*minVtx, vtxBegin[1]);	//	ŏ擪2ԖڂƓւ
	}

	//	ŏ̕ӂ
	lines.end->Clear();
	lines.end->vtx[0] = vtxBegin[1];
	lines.end->vtx[1] = vtxBegin[0];
	lines.end->CalcNormal();
	lines.end++;
	
	//	\čŏ̓ʑʑ̂ɂD
	*lines.end = *lines.begin;
	lines.end->Reverse();
	lines.begin->neighbor[0] = lines.end;
	lines.begin->neighbor[1] = lines.end;
	lines.end->neighbor[0] = lines.begin;
	lines.end->neighbor[1] = lines.begin;
	lines.end++;
	//	gp_𒸓_Xg͂D
	vtxBegin += 2;
	nLines = 2;
	return true;
}

/**	curƁC̖ʂԉ_ top 󂯎C
	curƂ̎͂̕ӂ폜Cʕtop܂߂D
	end[-1], end[-2]VɍꂽӂɂȂD	*/
template <class TVtx>
void CDQHLines<TVtx>::CreateCone(CDQHLine* cur, TVtx* top){
	cur->deleted = true;							//	cur͍폜
	nLines --;
	//	ׂ̕ӂĂāC_݂ӂ͍폜D
	CDQHLine* horizon[2];
	for(int i=0; i<2; ++i){
		horizon[i] = cur->neighbor[i];
		while(horizon[i]->Visible(top) && nLines>1){
			horizon[i]->deleted = true;
			nLines --;
			horizon[i] = horizon[i]->neighbor[i];
		}
	}
	//	폜Ȃӂƒ_̊ԂɕӂD
	//	horizon[0]̕
	end->Clear();
	horizon[0]->neighbor[1] = end;
	end->vtx[0] = horizon[0]->vtx[1];
	end->vtx[1] = top;
	end->neighbor[0] = horizon[0];
	end->neighbor[1] = end+1;
	end->CalcNormal();
	end++;
	nLines ++;
	//	horizon[1]̕
	end->Clear();
	horizon[1]->neighbor[0] = end;
	end->vtx[0] = top;
	end->vtx[1] = horizon[1]->vtx[0];
	end->neighbor[0] = end-1;
	end->neighbor[1] = horizon[1];
	end->CalcNormal();
	end++;
	nLines ++;
}	
/**	ԉ̒_D炻𒸓_Xg͂	*/
template <class TVtx>
bool CDQHLines<TVtx>::FindFarthest(CDQHLine* line){
#if 0	
	TVtx** maxVtx=NULL;
	double maxDist = epsilon;
	for(TVtx** it=vtxBegin; it!= vtxEnd; ++it){
		double dist = line->CalcDist(*it);
		if (dist > maxDist){
			maxDist = dist; 
			maxVtx = it;
		}
	}
	if (maxVtx){
		std::swap(*vtxBegin, *maxVtx);
		vtxBegin++;
#ifdef _DEBUG
		if (	vtxBegin[-1]->GetPos() == line->vtx[0]->GetPos()
			||	vtxBegin[-1]->GetPos() == line->vtx[1]->GetPos()){
			DSTR << "Error: same position." << std::endl;
			for(int i=0; i<2; ++i){
				DSTR << "V" << i << ": " << line->vtx[i]->GetPos() << std::endl;
			}
			DSTR << "P : " << vtxBegin[-1]->GetPos() << std::endl;
			assert(0);
		}
#endif
		return true;
	}
	return false;
#endif
	
	TVtx** maxVtx=NULL;
	double maxDist = epsilon;

	for(TVtx** it=vtxBegin; it!= vtxEnd; ++it){
		double dist = line->CalcDist(*it);
		if (dist > maxDist){
			maxDist = dist; 
			maxVtx = it;
		}
	}
	if (maxVtx){
		if (maxVtx[0]->GetPos() == line->vtx[0]->GetPos()
			||	maxVtx[0]->GetPos() == line->vtx[1]->GetPos()){
				return false;
		} else {
			std::swap(*vtxBegin, *maxVtx);
			vtxBegin++;
			
#ifdef _DEBUG
			if (vtxBegin[-1]->GetPos() == line->vtx[0]->GetPos()
				||	vtxBegin[-1]->GetPos() == line->vtx[1]->GetPos()){
			
				DSTR << "Error: same position." << std::endl;
				for(int i=0; i<2; ++i){
					DSTR << "V" << i << ": " << line->vtx[i]->GetPos() << std::endl;
				}
				DSTR << "P : " << vtxBegin[-1]->GetPos() << std::endl;
				assert(0);
			}
#endif
			return true;
		}
	}
	return false;
	
}
/*	O  ̏ɕׂD
ȌI聁̎n܂肪 inner	*/
template <class TVtx>
TVtx** CDQHLines<TVtx>::DivideByPlaneR(CDQHLine* plane, TVtx** start, TVtx** end){
	double INNER_DISTANCE = epsilon * plane->dist;
	while(start != end){
		double d = -plane->CalcDist(*start);
		if (d <= INNER_DISTANCE){	//	̏ꍇ͌Ɉړ
			-- end;
			std::swap(*end, *start);
		}else{
			++ start;
		}
	}
	return start;
}
template <class TVtx>
TVtx** CDQHLines<TVtx>::DivideByPlane(CDQHLine* plane, TVtx** start, TVtx** end){
	double INNER_DISTANCE = epsilon * plane->dist;
	while(start != end){
		double d = plane->CalcDist(*start);
		if (d <= INNER_DISTANCE){	//	̏ꍇ͌Ɉړ
			-- end;
			std::swap(*end, *start);
		}else{
			++ start;
		}
	}
	return start;
}
/**	̖ʂɑ΂鏈sDԉ̒_C
n𒲂ׁCR[C̒_͂D*/
template <class TVtx>
void CDQHLines<TVtx>::TreatPlane(CDQHLine* cur){
	//	ԉ̒_̒T
	if (!FindFarthest(cur)) return;
	HULL_DEBUG_EVAL(
		DSTR << "Farthest:" << vtxBegin[-1]->GetPos();
		DSTR << " cmp:" << vtxBegin[-1]->GetPos()*cur->normal << " > " << cur->dist << std::endl;
		if (cur->dist < 0){
			DSTR << "faceDist:" << cur->dist << std::endl;
		}
	)
	//	V_œʕD
	CreateCone(cur, vtxBegin[-1]);
	HULL_DEBUG_EVAL(
		if (!hor){
			DSTR << "No Horizon Error!!" << std::endl;
			assert(hor);
		}
	)
	//	ڂ(cur)ƐVȕ(end-2,end-1)ɂĕ߂钸_vtxEnďɈړ
	TVtx** inner = DivideByPlaneR(cur, vtxBegin, vtxEnd);
	for(CDQHLine* it=end-2; it!=end; ++it){
		HULL_DEBUG_EVAL(
			std::cout << "Inner:";
			for(TVtx** v = inner; v != vtxEnd; ++v){
				std::cout << **v;
			}
			std::cout << std::endl;
		);
		inner = DivideByPlane(it, inner, vtxEnd);
	}
	HULL_DEBUG_EVAL(
		std::cout << "InnerFinal:";
		for(TVtx** v = inner; v != vtxEnd; ++v){
			std::cout << **v;
		}
		std::cout << std::endl;
	);
	vtxEnd = inner;
}

template <class TVtx>
std::ostream& operator << (std::ostream& os, const CDQHLines<TVtx>& pls){
	pls.Print(os);
	return os;
}
template <class CDQHVtx2DSample>
inline std::ostream& operator << (std::ostream& os, const TYPENAME CDQHLines<CDQHVtx2DSample>::TVtxs& f){
	f.Print(os); return os;
}

inline void CDQHVtx2DSample::Print(std::ostream& os) const {
//		os << Vtx();
		os << id_ << " ";
}
inline std::ostream& operator << (std::ostream& os, const CDQHVtx2DSample& f){
	f.Print(os); return os;
}

}

#endif
