/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef CDQUICKHULL3D_H
#define CDQUICKHULL3D_H

#include <Base/Affine.h>
#include <Base/TQuaternion.h>
#include <iosfwd>
#include <vector>

#define HULL_EPSILON 1e-6f	///<	QuickHullňŏ̐l
#define HULL_INFINITE 1e8f	///<	QuickHullňő̐l

//#define HULL_DEBUG		//	fobOo
#ifdef HULL_DEBUG
 #define HULL_DEBUG_EVAL(x) x
#else
 #define HULL_DEBUG_EVAL(x)
#endif

namespace Spr{;

///	QuickHullō
template <class TVtx>
class CDQHPlane{
public:
	Vec3d normal;			///<	ʂ̖@
	double dist;			///<	ʂ̌_̋
	
	TVtx* vtx[3];			///<	ʂ\钸_
	CDQHPlane* neighbor[3];	///<	ׂ̖ vtx[0]-vtx[1] ׂ̗ neighbor[0]
	bool deleted;			///<	폜ꂽʂtrue
	void Clear();			///<	NADgOɌĂԁD
	bool Visible(TVtx* p);	///<	_ v \邩ǂ
	///	v̒_ԍԂ(0..2Ԃ)DȂ3ԂD
	int GetVtxID(TVtx* v);
	void CalcNormal();		///<	@xNgƋvZD
	void Reverse();			///<	\ЂԂD		
	///	_Ƃ̋vZDxlĈԋ߂_ŌvZD
	double CalcDist(TVtx* v);
	///	fobOp\
	void Print(std::ostream& os) const;
};

///	ʂ̃obt@
template <class TVtx>
class CDQHPlanes{
public:
	TVtx** vtxBeginInput;	///<	cĂ钸_̐擪
	TVtx** vtxEndInput;		///<	cĂ钸_̍Ō̎
	///	_Vector
	class TVtxs: public std::vector<TVtx*>{
	public:
		void Print(std::ostream& os) const{
			for(typename TVtxs::const_iterator it = begin(); it != end(); ++it){
				(*it)->Print(os);
			}
		}
 	};
	typedef CDQHPlane<TVtx> CDQHPlane;

	CDQHPlane* buffer;		///<	obt@ւ̃|C^ new D
	int len;			///<	obt@̒
	CDQHPlane* begin;		///<	ŏ̖
	CDQHPlane* end;		///<	Ō̖ʂ̎
	TVtx** vtxBegin;	///<	cĂ钸_̐擪
	TVtx** vtxEnd;		///<	cĂ钸_̍Ō̎
	int nPlanes;		///<	ʂ̐
	unsigned size();	///<	gpς݃obt@̃TCY
	CDQHPlanes(int l);
	void Clear();
	~CDQHPlanes();
	/**	be܂ł̒_ʕDgp_bvtxBeginC
		gpȂ_́CvtxBegineɈړD	
		beginend͒_3܂ޖʂɂȂD̖ʂʕɎgꂽ
		 CDQHPlane::deleted  false ɂȂĂD	*/
	void CreateConvexHull(TVtx** b, TVtx** e);
	void Print(std::ostream& os) const;

private:
	/**	ŏ̓ʑʑ́2̎Op`(\)D
		ł邾傫3p`ƌǂ̂ŁCxőCxŏC
		2_钼ԉ_3p`D
		gp_́CvtxBegin̎OɈړāC_Xg͂D
	*/
	void CreateFirstConvex();
	/**	horizon D cur ʁCvtx V_D
	rv horizonӂɎ3p`1ԂD*/
	void FindHorizon(TVtx*& rv, CDQHPlane* cur, TVtx* vtx);
	/**	_horizon̊ԂɃR[D*/
	void CreateCone(TVtx* firstVtx, TVtx* top);
	/**	ԉ̒_D炻𒸓_Xg͂	*/
	bool FindFarthest(CDQHPlane* plane);
	/*	O  ̏ɕׂD
		ȌI聁̎n܂肪 inner	*/
	TVtx** DivideByPlaneR(CDQHPlane* plane, TVtx** start, TVtx** end);
	TVtx** DivideByPlane(CDQHPlane* plane, TVtx** start, TVtx** end);
	/**	̖ʂɑ΂鏈sDԉ̒_C
		n𒲂ׁCR[C̒_͂D*/
	void TreatPlane(CDQHPlane* cur);
};

///	_NX̗
class CDQHVtx3DSample{
public:
	///@name QuickHullgp郁oDK{D
	//@{
	/**	QuickHullASYp[NGAD
		ԉ_猩ʂ폜Ǝc`̃GbW
		ڑĂ߂̃|C^D
		_ʂ̐ڑD	ʁ_͒_̕я番D	*/
	CDQHPlane<CDQHVtx3DSample>* horizon;
	///  _̈ʒu
	Vec3f GetPos() const;
	//@}
	///	ʒu̐ݒ
	void SetPos(Vec3f p);
public:
	Vec3f dir;		///<	_̌
	float dist;		///<	_̋
public:
	int id_;

	///	fobOp\
	void Print(std::ostream& os) const;
};

}

#endif
