/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef SCENE_H
#define SCENE_H
#include <SprFoundation.h>
#include "Object.h"
#include <set>
#include <map>

namespace Spr {;


class ObjectNamesLess{
public:
	bool operator () (const NamedObject* o1, const NamedObject* o2) const;
};
/**	OƃV[Ot̃IuWFNg̑Ή\
	OL[ɂZbgpӂCÖӐ̕ۏ؂ƃIuWFNg
	D	*/
class ObjectNames:public std::set<NamedObject*, ObjectNamesLess >{
public:
	typedef std::map<UTString, UTString> TNameMap;
	class ObjectKey:public NamedObject{
	public:
		ObjectKey();
		~ObjectKey();
		DEF_UTTYPEINFODEF(ObjectKey);
	};
	TNameMap nameMap;
	static ObjectKey key;
	void Print(std::ostream& os) const;

	///	OIuWFNg擾
	typedef std::pair<iterator, iterator> range_type;
	NamedObject* Find(UTString name, UTString cn="") const {
		key.name = name;
		iterator it = ((ObjectNames*)this)->find(&key);
		if (it == end()) return NULL;
		NamedObject* obj = *it;
		if (!cn.length()) return obj;
		if (obj->GetTypeInfo()->Inherit(cn.c_str())) return obj;
		return NULL;
	}
	/**	IuWFNg̒ǉC
		ÔȂIuWFNg͒ǉłȂD̏ꍇ false ԂD
		ǉɐ trueD łɓo^Ăꍇ false ԂD
		OԂꍇ́CǉIuWFNg̖OύXD	*/
	bool Add(NamedObject* obj);
	///	IuWFNg̍폜
	bool Del(NamedObject* key){
		iterator it = find(key);
		if (it==end()) return false;
		erase(it);
		return true;
	}
	///	IuWFNg̍폜
	bool Del(UTString name){
		NamedObject key;
		key.name = name;
		key.AddRef();
		iterator it = find(&key);
		key.DelRef();
		if (it==end()) return false;
		erase(it);
		return true;
	}
	///	ׂč폜
	void Clear();
};
inline std::ostream& operator << (std::ostream& os, const ObjectNames& ns){
	ns.Print(os); return os;
}

class SPR_DLL NameManager:public NamedObject, public NameManagerIfInit{
public:
	OBJECTDEF(NameManager, NamedObject);
protected:
	/*	OƃIuWFNg̑Ή\  */
	ObjectNames names;
	
	///@name	OԂ\邽߂̃c[
	//@{
	///
	typedef std::vector<NameManager*> NameManagers;
	///	q̖O
	NameManagers childManagers;
	//@}

public:
	NameManager();
	~NameManager();
	virtual void SetNameManager(NameManager* s);
	void AddChildManager(NameManager* c);
	void DelChildManager(NameManager* c);
	///	^ƖOIuWFNg擾
	template <class T> void FindObject(UTRef<T>& t, UTString name){
		T* p;
		FindObject(p, name);
		t = p;
	}
	template <class T> void FindObject(T*& t, UTString name){
		NamedObject* p = FindObject(name, GETCLASSNAMES(T));
		t = DCAST(T, p);
	}
	virtual NamedObjectIf* FindObject(UTString name, UTString cls="");
	typedef ObjectNames::iterator SetIt;
//	typedef std::pair<SetIt, SetIt> SetRange;
//	SetRange RangeObject(UTString n){ return names.Range(n); }
	
	ObjectNames::TNameMap& GetNameMap(){ return names.nameMap; }

	void Clear();

	///	fobOp
	void Print(std::ostream& os) const;
	friend class NamedObject;

	//	ʂ͌ĂяoȂCO̓֐D
	NamedObjectIf* FindObjectExact(UTString name, UTString cls="");
	NamedObjectIf* FindObjectFromDescendant(UTString name, UTString cls="");
	NamedObjectIf* FindObjectFromAncestor(UTString name, UTString cls="");	
};

/**	V[̊{NX	*/
class SPR_DLL Scene:public NameManager, public SceneIfInit{
	OBJECTDEF(Scene, NameManager);
public:
	///	RXgN^
};

class SPR_DLL Sdk:public NameManager, public SdkIfInit{
	UTRef<TypeInfoManager> typeInfoManager;	//	typeInfo/IfInfosdkOɏ邱ƂB
protected:
	friend struct SdkIf;
	static std::vector< UTRef<FactoryBase> > sdkFactories;
public:
	OBJECTDEF(Sdk, NameManager);
	static void SPR_CDECL RegisterFactory(FactoryBase* sdkFactory);
public:
	Sdk();
	virtual ~Sdk(){}
};

}
#endif
