/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef UTLoadContext_H
#define UTLoadContext_H

#include <Foundation/Object.h>
#include <Foundation/UTTypeDesc.h>


namespace Spr{;


///	t@C}bv(̂Ƃt@C̃[hp)
class UTFileMap: public UTRefCount{
public:
	std::string name;	///<		t@C
	const char* start;	///<		}bvꂽt@C̐擪
	const char* end;	///<		}bvꂽt@C̏I[
	const char* curr;	///<		݂̈ʒu
	/// RXgN^
	UTFileMap():start(NULL), end(NULL), curr(NULL){}
	///	fXgN^
	virtual ~UTFileMap(){}
	/// t@C }bsO		
	virtual bool Map(const UTString fn)=0;
	/// t@C A}bsO
	virtual void Unmap()=0;
	///	[hłԂȂtrue
	bool IsGood();
};

class UTLoadContext;
class UTLoadedData;
class UTLoadedDatas: public std::vector< UTLoadedData* >{
};
class UTLoadedDataRefs: public std::vector< UTRef<UTLoadedData> > {
};
class UTNameManagerForData;
/**	t@C[h̍ۂDOMm[ĥ悤Ȗm[h
	t@CObjectDescǂݏoCt@Cɏ񂾂肷邽߂̃f[^D
	ObjectDesc ւ̃|C^(data)  ^ (type) D
	̊ǗsD	*/
class UTLoadedData: public UTRefCount{
public:
	typedef std::map<UTString, UTString> Attributes;
protected:
	 Attributes attributes;			///<	

public:
	UTRef<UTFileMap> fileInfo;		///<	t@C
	const char* filePos;			///<	t@Cł̈ʒu
	UTLoadedData* parent;			///<	em[h
	UTLoadedDataRefs children;		///<	qm[h
	UTLoadedDatas linkFrom;			///<	Qƌm[h
	UTLoadedDatas linkTo;			///<	QƐm[h
	UTTypeDesc* type;				///<	^
	void* data;						///<	[hfBXNv^
	UTString str;					///<	[he̕D^񂪂ȂꍇɎgD
	bool haveData;					///<	dataL邩ǂ

	///	typeNameManager̔hNX̏ꍇCDatapNameManger
	UTRef<UTNameManagerForData> nameMan;	
	UTLoadedData* man;				///<	̃f[^̖OǗNameMangerf[^
	
	///	̃f[^ꂽIuWFNg
	ObjectIfs loadedObjects;

	UTLoadedData(UTLoadContext* fc, UTTypeDesc* t, void* data=NULL);
	~UTLoadedData();
	void AddLink(UTLoadedData* to);	///<	QƂ̒ǉ
	void AddChild(UTLoadedData* c);	///<	qm[h̃f[^̒ǉ
	void SetupNameManager();
	void SetType(UTTypeDesc* t);
	void SetName(UTString n);
	UTString GetName() const;
	void SetAttribute(UTString key, UTString val);
	UTString GetAttribute(UTString key) const;
	bool HasAttribute(UTString key) const;
	friend class UTNameManagerForData;
	bool operator < (const UTLoadedData& d2) const {
		return GetName() < d2.GetName();
	}
	UTLoadedData* FindAncestor(UTString tn);
	UTLoadedData* FindDescendant(UTString tn);
	UTLoadedData* FindLinkAncestor(UTString tn);
	UTLoadedData* FindLinkDescendant(UTString tn);
	void EnumLinkAncestor(std::vector<UTLoadedDatas>& res, UTString tn);
	void EnumLinkDescendant(std::vector<UTLoadedDatas>& res, UTString tn);
	void Print(std::ostream& os);
};

///	UTLoadedData p̖OǗ
class UTNameManagerForData: public UTRefCount{
public:

	typedef std::set<UTLoadedData*, UTContentsLess<UTLoadedData*> > DataSet;
	typedef std::map<UTString, UTString> NameMap;
	///@name	OԂ\邽߂̃c[
	//@{
	///
	typedef std::vector<UTNameManagerForData*> NameManagers;
	///	̃f[^
	UTLoadedData* data;
	///	qO
	NameManagers childManagers;
	///	eO
	UTNameManagerForData* parent;
	//@}

	UTLoadedData* FindData(UTString name, UTString cls="");
protected:
	NameMap nameMap;
	DataSet dataSet;
public:
	UTNameManagerForData();

	///	UTLoadedData𖼑O\ɒǉ
	bool AddData(UTLoadedData* data);

	/**	O̕ϊiOԂꍇCm[h̖Oϊ邪C
		̊֐́C̖Oϊ̖O̕ϊD	
		@param n ̖O
		@return ϊ̖O
	*/
	UTString MapName(UTString n);

protected:
	UTLoadedData* FindDataFromAncestor(UTString name, UTString cls);
	UTLoadedData* FindDataFromDescendant(UTString name, UTString cls);
	UTLoadedData* FindDataExact(UTString name, UTString cls);
	UTLoadedData* SearchSet(UTString name, UTString cls);
};

class UTLoadContext;
///	[hɏ邽߂ɁA[hɐA[hɎsD
class UTLoadTask:public NamedObject{
public:
	OBJECTDEF_NOIF(UTLoadTask, NamedObject);
	virtual ~UTLoadTask(){}
	virtual void Execute(UTLoadContext* ctx){};
};

///	[hɃN邽߂̃^XNDʂ͎gȂD
class UTLinkTask:public UTLoadTask{
public:
	ObjectIf* linkFrom;
	ObjectIf* linkTo;
	UTLinkTask(ObjectIf* from, ObjectIf* to):linkFrom(from), linkTo(to){}
	virtual void Execute(UTLoadContext* ctx){
		linkFrom->AddChildObject(linkTo);
	};
	virtual ~UTLinkTask(){}
};

///	[h̏s߂̃^XNXg
class UTLoadTasks:public std::vector< UTRef<UTLoadTask> >{
public:
	void Execute(UTLoadContext* ctx);
};

typedef std::pair<UTRef<ObjectIf>, UTRef<ObjectIf> > UTPairObject;
typedef std::map<UTRef<ObjectIf>, UTRef<ObjectIf> >  UTMapObject;

class UTLoadHandlerDb;
/**	t@C[hɎgpReLXg
	t@C[hۂ́Cf[^m[hƂɃ[hāC
	IuWFNg邽߂̃fBXNv^\(PHSolidDescȂ)
	܂[hD
	̂ƁCIuWFNg𐶐D	*/
class UTLoadContext{
public:	
	//--------------------------------------------------------------------------
	//	ϐ
	///	[h̃t@C̖OƒgDt@CincludeɔstackɂȂĂD
	UTStack< UTRef<UTFileMap> > fileMaps;
	///	݃[h̃IuWFNgDlXgIuWFNgɔăX^bNɂȂĂD
	ObjectIfs objects;
	///	X^bNɍŏɋl܂ꂽIuWFNgt@C̈ԊO[g̃IuWFNg̋L^D
	ObjectIfs rootObjects;
	///	[hfBXNv^̃X^bNDlXggݗČ^ɔăX^bNɂȂĂD
	UTStack< UTRef<UTLoadedData> > datas;
	struct LoadedDatas:public std::vector< UTRef<UTLoadedData> >{
		void Print(std::ostream& os);
	};
	///	[hfBXNv^̋L^D
	LoadedDatas loadedDatas;
	///	[hFITypedesc̃tB[ḧʒuDgݗČ^̃tB[hɔăX^bNɂȂĂD
	UTTypeDescFieldIts fieldIts;
	///	tÕX^bN
	UTStack<char> flags;

	///	typeDb ̃X^bN
	UTStack< UTRef<UTTypeDescDb> > typeDbs;
	///	handlerDb̃X^bN
	UTStack< UTRef<UTLoadHandlerDb> > handlerDbs;

	///	G[bZ[Wo͗p̃Xg[ cout Ƃ DSTR w肷D
	std::ostream* errorStream;
	///	f[^̃Nł邽߂̋L^D
	UTLoadTasks dataLinks;
	///	t@XŃN邽߂̋L^D
	UTLoadTasks links;
	///	[hƃNIĂ珈^XN
	UTLoadTasks postTasks;
	// }bvIuWFNgDm[hԂŊ֌WꍇɎgpD
	UTMapObject mapObj;	
	//	f[^p̃[gNameManager̂߂̃IuWFNg
	UTRef<UTLoadedData> rootNameManagerForData;

	//---------------------------------------------------------------------------
	///	RXgN^
	UTLoadContext();
	//@name [_Cnh߂̃[eBeB
	//@{
	///	G[bZ[W̏óDpost@CƍsԍɕϊD
	void ErrorMessage(UTFileMap* info, const char* pos, const char* msg);
	///	bZ[W̍쐬Dpost@CƍsԍɕϊD
	void Message(UTFileMap* info, const char* pos, const char* msg);
	///	[hłԂȂtrue
	bool IsGood();
	///	type鏀(typeZbgCXXDescpӂ)
	void NodeStart(UTString tn, UTLoadedData::Attributes* attrs=NULL);
	////
	void NodeEnd();
	///
	void WriteString(std::string v);
	///
	void WriteNumber(double d);
	///
	void WriteBool(bool b);
	///
	void AddDataLink(std::string ref, const char* pos);	
	//@}
	//@name f[^IuWFNgƂ̃[eBeB
	//@{
	///	f[^IuWFNg쐬
	ObjectIf* CreateObject(const IfInfo* info,  const void* data, UTString name="");
	///	
	UTLoadedData* FindAncestor(UTString tn);
	///	
	UTLoadedData* FindDescendant(UTString tn);
	///
	UTLoadedData* FindLinkAncestor(UTString tn);
	///
	UTLoadedData* FindLinkDescendant(UTString tn);

	//@}
	void LinkData();
	///
	void LinkNode();
	///
	void PostTask();
	///	t@C}bv쐬ăX^bN(fileMaps)ɐς
	virtual void PushFileMap(const UTString fn)=0;
	/**	[hm[h̃O[vo^BO[vXy[Xŋ؂ĎwB
		FResisterGroupToDb("Foundation Physics Graphics Framework OldSpringhead");
	*/
	void RegisterGroupToDb(const char* gp);

	  virtual ~UTLoadContext(){}
protected:
	void LinkNode(UTLoadedData* ld);
};

}

#endif	// UTLoadContext_H
