/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#include "Graphics.h"
#include "GRMesh.h"


namespace Spr{;
IF_OBJECT_IMP(GRMesh, GRVisual);

GRMesh::GRMesh(const GRMeshDesc& desc):GRMeshDesc(desc){
	render = NULL;
}
GRMesh::~GRMesh(){
	for (unsigned int id=0; id<list.size(); ++id){
		if (list[id]) render->ReleaseList(list[id]);
	}
}
void GRMesh::CreateList(GRRenderIf* r){
	for (unsigned int id=0; id<list.size(); ++id){
		if (list[id])	render->ReleaseList(list[id]);
	}
	list.clear();
	render = r;
	unsigned int vtxsize = max(max(positions.size(), normals.size()), max(colors.size(), texCoords.size()));

	if (texCoords.size() && normals.size() && colors.size()){
		std::vector<GRVertexElement::VFT2fC4fN3fP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<originalFaces.size(); ++i){ vtx[originalFaces[i]].n = normals[faceNormals[i]]; }
		for (unsigned i=0; i<colors.size(); ++i){ vtx[i].c = colors[i];	}
		for (unsigned i=0; i<texCoords.size(); ++i){ vtx[i].t = texCoords[i]; }
		render->SetVertexFormat(GRVertexElement::vfT2fC4fN3fP3f);
		this->CreateListElement(&*vtx.begin());
	}else if (texCoords.size() && normals.size()){
		std::vector<GRVertexElement::VFT2fN3fP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<originalFaces.size(); ++i){ vtx[originalFaces[i]].n = normals[faceNormals[i]]; }
		for (unsigned i=0; i<texCoords.size(); ++i){ vtx[i].t = texCoords[i]; }
		render->SetVertexFormat(GRVertexElement::vfT2fN3fP3f);
		this->CreateListElement(&*vtx.begin());
	}else if (texCoords.size() && colors.size()){
		std::vector<GRVertexElement::VFT2fC4bP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<colors.size(); ++i){ 
			vtx[i].c = ((unsigned char)(colors[i].x*255)) |
			 		   ((unsigned char)(colors[i].y*255) << 8) |
					   ((unsigned char)(colors[i].z*255) << 16) |
					   ((unsigned char)(colors[i].w*255) << 24);
		}
		for (unsigned i=0; i<texCoords.size(); ++i){ vtx[i].t = texCoords[i]; }
		render->SetVertexFormat(GRVertexElement::vfT2fC4bP3f);
		this->CreateListElement(&*vtx.begin());
	}else if (normals.size() && colors.size()){
		std::vector<GRVertexElement::VFC4fN3fP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<originalFaces.size(); ++i){ vtx[originalFaces[i]].n = normals[faceNormals[i]]; }
		for (unsigned i=0; i<colors.size(); ++i){ vtx[i].c = colors[i];	}
		render->SetVertexFormat(GRVertexElement::vfC4fN3fP3f);
		this->CreateListElement(&*vtx.begin());
	}else if (normals.size()){
		std::vector<GRVertexElement::VFN3fP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<originalFaces.size(); ++i){ vtx[originalFaces[i]].n = normals[faceNormals[i]]; }
		render->SetVertexFormat(GRVertexElement::vfN3fP3f);
		this->CreateListElement(&*vtx.begin());
	}else if (texCoords.size()){
		std::vector<GRVertexElement::VFT2fP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<texCoords.size(); ++i){ vtx[i].t = texCoords[i]; }
		render->SetVertexFormat(GRVertexElement::vfT2fP3f);
		this->CreateListElement(&*vtx.begin());
	}else if (colors.size()){
		std::vector<GRVertexElement::VFC4bP3f> vtx;
		vtx.resize(vtxsize);
		for (unsigned i=0; i<positions.size(); ++i){ vtx[i].p = positions[i]; }
		for (unsigned i=0; i<colors.size(); ++i){ 
			vtx[i].c = ((unsigned char)(colors[i].x*255)) |
			 		   ((unsigned char)(colors[i].y*255) << 8) |
					   ((unsigned char)(colors[i].z*255) << 16) |
					   ((unsigned char)(colors[i].w*255) << 24);
		}
		this->CreateListElement(&*vtx.begin());
	}else{
		render->SetVertexFormat(GRVertexElement::vfP3f);
		this->CreateListElement(&*positions.begin());
	}
}

/// }eACfbNXꍇ́AꖈɂP̃fBXvCXgƂāAo^D
void GRMesh::CreateListElement(void *vtx){
	int base = 0;		// fBXvCXg̎ʎq
	// Xt@C materialList w肪Ȃꍇ
	//if (!materialList.size()){		
	if (materialList.empty()){
		base = render->CreateIndexedList(GRRenderIf::TRIANGLES, &*faces.begin(), vtx, faces.size());
		list.push_back(base);		
	}
	// Xt@C materialList w肪ꍇAmaterialPʂ Display List 
	else{							
		unsigned int first	= 0;				// elementIndex[]ɂAE first frame
		unsigned int end	= first+1;			// elementIndex[]ɂAE end frame
		
		// ()Op`̖ʂ̐[v
		while (end < elementIndex.size()){		
			// Xt@C̎wŁA`ʂ̐A}eAݒ肳ꂽʂ̐Ȃǂ̃`FbN
			if (materialList.size() > elementIndex[end]){ 
				// }eACfbNXǂr
				if (materialList[elementIndex[first]] == materialList[elementIndex[end]]){
					end++;							// 1炵āA܂firstƔr
					if (end >= elementIndex.size()){		// elementIndexz̈ԍŌ̗vfǂ̃`FbN					
						// Kp}eAIDAmaterialƂėpӂĂ邩H
						if (materialList[elementIndex[first]] < (int)material.size()){
							base = render->CreateIndexedList(material[materialList[elementIndex[first]]]->Cast(), 	
																GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, (end-first)*3);
						}else{
							base = render->CreateIndexedList(GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, (end-first)*3);							
						}
						list.push_back(base);
						break;
					}
				}else{
					// Kp}eAIDAmaterialƂėpӂĂ邩H
					if (materialList[elementIndex[first]] < (int)material.size()) {		
						base = render->CreateIndexedList(material[materialList[elementIndex[first]]]->Cast(), 					
																GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, (end-first)*3);
					}else{
						base = render->CreateIndexedList(GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, (end-first)*3);
					}
					list.push_back(base);
					
					first = end;
					end = end+1;
						
					if (end >= elementIndex.size()){			// elementIndexz̈ԍŌ̗vfǂ̃`FbN
						// Kp}eAIDAmaterialƂėpӂĂ邩H
						if (materialList[elementIndex[first]] < (int)material.size()) {		
							base = render->CreateIndexedList(material[materialList[elementIndex[first]]]->Cast(),  
																	GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, (end-first)*3);
						}else{
							base = render->CreateIndexedList(GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, (end-first)*3); 
						}	
						list.push_back(base);
						break;
					}
				}

			}else{
				/** Xt@C MeshMaterialList w肵ꍇA
					SĂ̒`ʂɑ΂āAĂ}eAݒĂ邪A
					ʂ̐A}eAݒ肳ꂽʂ̐Ȃꍇ́A
					ŌɃ}eAݒ肳ꂽʂƓ}eAA}eAݒ肳ĂȂʂɑ΂ēKpB*/
				// Kp}eAIDAmaterialƂėpӂĂ邩H
				if (materialList[elementIndex[first]] < (int)material.size()) {	
					base = render->CreateIndexedList(material[materialList[elementIndex[first]]]->Cast(), 
														GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, faces.size()-first*3);
				}else{
					base = render->CreateIndexedList(GRRenderIf::TRIANGLES, &*faces.begin()+first*3, vtx, faces.size()-first*3);				
				}
				list.push_back(base);
				break;
					
			}
		}	
	}
}
void GRMesh::Render(GRRenderIf* r){
	if (r!=render || !list.size()) CreateList(r);
	for (unsigned int id=0; id<list.size(); ++id){
		render->DrawList(list[id]);
	}
}
void GRMesh::Rendered(GRRenderIf* r){
}
bool GRMesh::AddChildObject(ObjectIf* o){			
	GRMaterial* m = DCAST(GRMaterial, o);
	if (m){
		material.push_back(m);
		return GetNameManager()->AddChildObject(m->Cast());
	}
	return false;
}
size_t GRMesh::NChildObject() const {
	return material.size();
}
ObjectIf* GRMesh::GetChildObject(size_t pos){
	if (pos < material.size()) return material[pos]->Cast();
	return NULL;
}

}
