/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
/**
 *	@file GRRender.h
 *	@brief OtBbNX_[̊{NX@@
 */
#ifndef GRRENDER_H
#define GRRENDER_H

#include <SprGraphics.h>
#include <Foundation/Scene.h>
#include "GRFrame.h"
#include "IfStubGraphics.h"

namespace Spr{;

class GRCamera:public GRVisual, public GRCameraIfInit, public GRCameraDesc{
public:
	OBJECTDEF(GRCamera, GRVisual);
	ACCESS_DESC(GRCamera);
	UTRef<GRFrameIf> frame;
	GRCamera(const GRCameraDesc& desc=GRCameraDesc()):GRCameraDesc(desc), frame(NULL){}
	virtual size_t NChildObject() const ;
	virtual ObjectIf* GetChildObject(size_t pos);
	virtual GRFrameIf* GetFrame(){ return frame; }
	virtual void SetFrame(GRFrameIf* fr){ frame = fr; }
	virtual bool AddChildObject(ObjectIf* o);
	virtual void Render(GRRenderIf* render);	
};

class GRLight :public GRVisual, public GRLightIfInit, public GRLightDesc{
public:
	OBJECTDEF(GRLight, GRVisual);
	ACCESS_DESC(GRLight);
	GRLight(const GRLightDesc& desc = GRLightDesc()):GRLightDesc(desc){}
	virtual void Render(GRRenderIf* render);
	virtual void Rendered(GRRenderIf* render);
};

/**	@brief	OtBbNX̍ގ */
class GRMaterial :public GRVisual, public GRMaterialIfInit, public GRMaterialDesc{
public:
	OBJECTDEF(GRMaterial, GRVisual);
	ACCESS_DESC(GRMaterial);
	GRMaterial(const GRMaterialDesc& desc=GRMaterialDesc()):GRMaterialDesc(desc){}
	GRMaterial(Vec4f a, Vec4f d, Vec4f s, Vec4f e, float p){
		ambient = a;
		diffuse = d;
		specular = s;
		emissive = e;
		power = p;
	}
	GRMaterial(Vec4f c, float p){
		ambient = diffuse = specular = emissive = c;
		power = p;
	}
	GRMaterial(Vec4f c){
		ambient = diffuse = specular = emissive = c;		
	}
	/**	W()vf́AAt@l(0.0`1.0œx\). 1.0s\.
		materialW()vf𔻒肵āAŝÂ𔻒肷. 
		ȃIuWFNg`ƂÂ̂珇Ԃɕ`悵ȂƁAӐ}ɔʂƂȂ. */
	bool IsOpaque() const {		
		return ambient.W() >= 1.0 && diffuse.W() >= 1.0 && specular.W() >= 1.0 && emissive.W() >= 1.0;
	}
	void Render(GRRenderIf* render);
	//virtual bool AddChildObject(ObjectIf* o);
};

/**	@class	GRRenderBase
    @brief	OtBbNX_[/foCX̊{NX@ */
class GRRenderBase: public Object, public GRRenderBaseIfInit{
public:
	OBJECTDEF_ABST(GRRenderBase, Object);
	///	r[|[g̐ݒ
	virtual void SetViewport(Vec2f pos, Vec2f sz){}
	///	obt@NA
	virtual void ClearBuffer(){}
	///	_O̊JnOɌĂԊ֐
	virtual void BeginScene(){}
	///	_ȌIɌĂԊ֐
	virtual void EndScene(){}
	///	Jg̎_safvŒu
	virtual void SetViewMatrix(const Affinef& afv){}
	///	Jg̓esafpŒu
	virtual void SetProjectionMatrix(const Affinef& afp){}
	///	Jg̓es擾
	virtual void GetProjectionMatrix(const Affinef& afp){}
	///	Jg̃fsafwŒu
	virtual void SetModelMatrix(const Affinef& afw){}
	///	Jg̃fsɑ΂afw|
	virtual void MultModelMatrix(const Affinef& afw){}
	///	Jg̃fsfsX^bN֕ۑ
	virtual void PushModelMatrix(){}
	///	fsX^bNoAJg̃fsƂ
	virtual void PopModelMatrix(){}
	/// uhϊs̑Svf폜
	virtual void ClearBlendMatrix(){}
	/// uhϊsݒ肷
	virtual bool SetBlendMatrix(const Affinef& afb, unsigned int id=0){return 0;}	
	///	_tH[}bg̎w
	virtual void SetVertexFormat(const GRVertexElement* e){}
	///	_VF[_[̎w	APIDȂǗv 2006.6.7 hase
	virtual void SetVertexShader(void* shader){}
	///	_w肵ăv~eBu`
	virtual void DrawDirect(GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0){}
	///	_ƃCfbNXw肵ăv~eBu`
	virtual void DrawIndexed(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0){}
 	///	_̐Ƃ̔zw肵āCv~eBu`
	virtual void DrawArrays(GRRenderBaseIf::TPrimitiveType ty, GRVertexArray* arrays, size_t count){}
 	///	CfbNXƒ_̐Ƃ̔zw肵āCv~eBu`
	virtual void DrawArrays(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, GRVertexArray* arrays, size_t count){}
	
	///	_CNg`ɂ DiplayList ̍쐬
	virtual int CreateList(GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0){return 0;}
	virtual int CreateList(GRMaterialIf* mat, unsigned int texid, 
						   GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0){return 0;}
	/// IuWFNg DisplayList ̍쐬
	virtual int CreateList(float radius, int slices, int stacks){return 0;}
	virtual int CreateList(GRMaterialIf* mat, float radius, int slices, int stacks){return 0;}
	///	CfbNX`ɂ DiplayList ̍쐬
	virtual int CreateIndexedList(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0){return 0;}
	virtual int CreateIndexedList(GRMaterialIf* mat, 
								  GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0){return 0;}
	/// CfbNX`ɂVF[_Kp DisplayList ̍쐬iSetVertexFormat()  SetShaderFormat() ̌ɌĂԁj
	virtual int CreateShaderIndexedList(GRHandler shader, void* location, 
									GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0){return 0;}	
	///	DisplayList̕\
	virtual void DrawList(int i){}
	///	DisplayList̉
	virtual void ReleaseList(int i){}
	///	2eLXg̕`@@ Windows(VC)ł̂fontT|[gÅłfontw肵ĂpȂB
	virtual void DrawFont(Vec2f pos, const std::string str, const GRFont& font=0){}
	///	3eLXg̕`@@ Windows(VC)ł̂fontT|[gÅłfontw肵ĂpȂB	
	virtual void DrawFont(Vec3f pos, const std::string str, const GRFont& font=0){}
	///	`̍ގ̐ݒ
	virtual void SetMaterial(const GRMaterialDesc& mat){}
	virtual void SetMaterial(const GRMaterialIf* mat){}
	///	`悷_Ȇ̐ݒ
	virtual void SetLineWidth(float w){}
	///	X^bNPush
	virtual void PushLight(const GRLightDesc& light){}
	virtual void PushLight(const GRLightIf* light){}
	///	X^bNPop
	virtual void PopLight(){}
	///	fvXobt@ւ݂̏/֎~
	virtual void SetDepthWrite(bool b){}
	///	fvXeXgL/ɂ
	virtual void SetDepthTest(bool b){}
	///	fvXobt@@ɗp锻w肷
	virtual void SetDepthFunc(GRRenderBaseIf::TDepthFunc f){}
	/// At@ufBOL/ɂ
	virtual void SetAlphaTest(bool b){}
	///	At@ufBÕ[hݒ(SRC̍W, DESTW)
	virtual void SetAlphaMode(GRRenderBaseIf::TBlendFunc src, GRRenderBaseIf::TBlendFunc dest){}
	/// eNX`̃[hi߂lFeNX`IDj
	virtual unsigned int LoadTexture(const std::string filename){return 0;}
	/// VF[_̏
	virtual void InitShader(){}
	/// VF[_tH[}bg̐ݒ
	virtual void SetShaderFormat(GRShaderFormat::ShaderType type){}	
	/// VF[_IuWFNg̍쐬
	virtual bool CreateShader(std::string vShaderFile, std::string fShaderFile, GRHandler& shader){return 0;}
	/// VF[_IuWFNg̍쐬AGRDeviceGL::shaderւ̓o^i炩ShaderFileo^ĂKvj	
	virtual GRHandler CreateShader(){return 0;}
	/// VF[_̃\[XvOɓǂݍ݁AVF[_IuWFNgƊ֘At
	virtual bool ReadShaderSource(GRHandler shader, std::string file){return 0;}	
	/// P[V̎擾iSetShaderFormat()ŃVF[_tH[}bgݒ肵ĂKvj
	virtual void GetShaderLocation(GRHandler shader, void* location){}
};

/**	@class	GRRender
    @brief	OtBbNX_[̊{NXifoCX̐؂蕪j@ */
class GRRender: public GRRenderBase, public GRRenderIfInit{
	OBJECTDEF(GRRender, GRRenderBase);
protected:
	UTRef<GRDeviceIf> device;		///<	foCX
	GRCameraDesc camera;			///<	J
	Vec2f viewportPos;				///<	r[|[g̍
	Vec2f viewportSize;				///<	r[|[g̃TCY
public:
#define REDIRECTIMP_GRRENDERBASE(ptr)																		\
	virtual void SetViewport(Vec2f p, Vec2f s){ ptr SetViewport(p, s); }									\
	virtual void ClearBuffer(){ ptr ClearBuffer(); }														\
	virtual void BeginScene(){ ptr BeginScene(); }															\
	virtual void EndScene(){ ptr EndScene(); }																\
	virtual void SetViewMatrix(const Affinef& afv){ ptr SetViewMatrix(afv); }								\
	virtual void SetProjectionMatrix(const Affinef& afp){ ptr SetProjectionMatrix(afp); }					\
	virtual void GetProjectionMatrix(const Affinef& afp){ ptr GetProjectionMatrix(afp); }					\
	virtual void SetModelMatrix(const Affinef& afw){ ptr SetModelMatrix(afw); }								\
	virtual void MultModelMatrix(const Affinef& afw){ ptr MultModelMatrix(afw); }							\
	virtual void PushModelMatrix(){ ptr PushModelMatrix(); }												\
	virtual void PopModelMatrix(){ ptr PopModelMatrix(); }													\
	virtual bool SetBlendMatrix(const Affinef& afb){ return ptr SetBlendMatrix(afb); }						\
	virtual bool SetBlendMatrix(const Affinef& afb, unsigned int id){ return ptr SetBlendMatrix(afb, id); }	\
	virtual void ClearBlendMatrix(){ ptr ClearBlendMatrix(); }												\
	virtual void SetVertexFormat(const GRVertexElement* f){ ptr SetVertexFormat(f); }						\
	virtual void DrawDirect(GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t ct, size_t st=0)			\
		{ ptr DrawDirect(ty, vtx, ct, st); }																\
	virtual void DrawIndexed(GRRenderBaseIf::TPrimitiveType ty,												\
		size_t* idx, void* vtx, size_t ct, size_t st=0)														\
		{ ptr DrawIndexed(ty, idx, vtx, ct, st); }															\
	virtual int CreateList(GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0)		\
		{ return ptr CreateList(ty, vtx, count, stride); }													\
	virtual int CreateList(GRMaterialIf* mat, unsigned int texid,											\
		GRRenderBaseIf::TPrimitiveType ty, void* vtx, size_t count, size_t stride=0)						\
		{ return ptr CreateList(mat, texid, ty, vtx, count, stride); }										\
	virtual int CreateList(float radius, int slices, int stacks)											\
		{ return ptr CreateList(radius, slices, stacks); }													\
	virtual int CreateList(GRMaterialIf* mat, float radius, int slices, int stacks)							\
		{ return ptr CreateList(mat, radius, slices, stacks); }												\
	virtual int CreateIndexedList(GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, 				\
		size_t count, size_t stride=0)																		\
		{ return ptr CreateIndexedList(ty, idx, vtx, count, stride); }										\
	virtual int CreateIndexedList(GRMaterialIf* mat,														\
		GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0)			\
		{ return ptr CreateIndexedList(mat, ty, idx, vtx, count, stride); }									\
	virtual int CreateShaderIndexedList(GRHandler shader, void* location, 									\
		GRRenderBaseIf::TPrimitiveType ty, size_t* idx, void* vtx, size_t count, size_t stride=0)			\
		{ return ptr CreateShaderIndexedList(shader, location, ty, idx, vtx, count, stride); }				\
	virtual void DrawList(int i){ ptr DrawList(i); }														\
	virtual void ReleaseList(int i){ ptr ReleaseList(i); }													\
	virtual void DrawFont(Vec2f pos, const std::string str){ ptr DrawFont(pos, str); }						\
    virtual void DrawFont(Vec3f pos, const std::string str){ ptr DrawFont(pos, str); }						\
	virtual void DrawFont(Vec2f pos, const std::string str, const GRFont& font)								\
		{ ptr DrawFont(pos, str, font); }																	\
	virtual void DrawFont(Vec3f pos, const std::string str, const GRFont& font)								\
		{ ptr DrawFont(pos, str, font); }																	\
	virtual void SetMaterial(const GRMaterialDesc& mat){ ptr SetMaterial(mat); }							\
	virtual void SetMaterial(const GRMaterialIf* mat)														\
		{ if(mat) ptr SetMaterial((const GRMaterialDesc&)*DCAST(GRMaterial, mat)); }						\
	virtual void SetLineWidth(float w){ ptr SetLineWidth(w); }												\
	virtual void PushLight(const GRLightDesc& light){ptr PushLight(light);}									\
	virtual void PushLight(const GRLightIf* light)															\
		{ if(light) ptr PushLight((const GRLightDesc&)*DCAST(GRLight, light)); }							\
	virtual void PopLight(){ptr PopLight(); }																\
	virtual void SetDepthWrite(bool b){ ptr SetDepthWrite(b); }												\
	virtual void SetDepthTest(bool b){ptr SetDepthTest(b); }												\
	virtual void SetDepthFunc(GRRenderBaseIf::TDepthFunc f){ ptr SetDepthFunc(f); }							\
	virtual void SetAlphaTest(bool b){ptr SetAlphaTest(b); }												\
	virtual void SetAlphaMode(GRRenderBaseIf::TBlendFunc src, GRRenderBaseIf::TBlendFunc dest)				\
		{ ptr SetAlphaMode(src, dest); }																	\
	virtual unsigned int LoadTexture(const std::string filename){ return ptr LoadTexture(filename); }		\
	virtual void InitShader(){ ptr InitShader(); }															\
	virtual void SetShaderFormat(GRShaderFormat::ShaderType type){ ptr SetShaderFormat(type); }				\
	virtual bool CreateShader(std::string vShaderFile, std::string fShaderFile, GRHandler& shader)			\
		{ return ptr CreateShader(vShaderFile, fShaderFile, shader); }										\
	virtual GRHandler CreateShader(){ return ptr CreateShader(); }											\
	virtual bool ReadShaderSource(GRHandler shader, std::string file)										\
		{ return ptr ReadShaderSource(shader, file); }														\
	virtual void GetShaderLocation(GRHandler shader, void* location)										\
		{ ptr GetShaderLocation(shader, location); }														\
	
	REDIRECTIMP_GRRENDERBASE(device->)
	
	///	foCX̐ݒ
	virtual void SetDevice(GRDeviceIf* dev){ device = dev; }
	///	fobO\
	virtual void Print(std::ostream& os) const;
	///	J̐ݒ
	void SetCamera(const GRCameraDesc& c);
	///	XN[TCYƃvWFNVs̐ݒ
	virtual void Reshape(Vec2f pos, Vec2f sz);
};

/**	@class	GRDevice
    @brief	OtBbNX`̎@ */
class GRDevice: public GRRenderBase, public GRDeviceIfInit{
public:
	OBJECTDEF_ABST(GRDevice, GRRenderBase);
	virtual void Init(){}
	virtual void SetMaterial(const GRMaterialDesc& mat){}
	virtual void SetMaterial(const GRMaterialIf* mat){
		if(mat) SetMaterial(*DCAST(GRMaterial, mat)); }
	virtual void PushLight(const GRLightDesc& light){}
    virtual void PushLight(const GRLightIf* light){
        if(light) PushLight(*DCAST(GRLight, light)); }
};
}
#endif
