/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#include "Physics.h"
#ifdef USE_HDRSTOP
#pragma hdrstop
#endif

using namespace std;
using namespace PTM;
namespace Spr{;

//----------------------------------------------------------------------------
// PHConstraint
IF_OBJECT_IMP_ABST(PHConstraint, SceneObject);
PHConstraint::PHConstraint(){
	solid[0] = solid[1] = NULL;
	f.clear();
	F.clear();
	bEnabled = true;
	bInactive[0] = true;
	bInactive[1] = true;
	bArticulated = false;
}
bool PHConstraint::AddChildObject(ObjectIf* o){
	PHSolid* s = DCAST(PHSolid, o);
	if(s){
		//PHSolids::iterator it = (PHSolids::iterator) scene->constraintEngine->solids.Find(s);
		//if(it == scene->constraintEngine->solids.end())
		//	return false;
		if(!solid[0]){
			solid[0] = s;
			return true;
		}
		if(!solid[1]){
			solid[1] = s;
			return true;
		}
	}
	return false;
}
size_t PHConstraint::NChildObject(){
	return (solid[0] ? 1 : 0) + (solid[1] ? 1 : 0);
}
ObjectIf* PHConstraint::GetChildObject(size_t i){
	return solid[i]->Cast();
}

void PHConstraint::SetDesc(const void* desc){
	const PHConstraintDesc& condesc = *(const PHConstraintDesc*)desc;
	Xj[0].r = condesc.poseSocket.Pos();
	Xj[0].q = condesc.poseSocket.Ori();
	Xj[1].r = condesc.posePlug.Pos();
	Xj[1].q = condesc.posePlug.Ori();
	bEnabled = condesc.bEnabled;
}

void PHConstraint::UpdateState(){
	// ̂̑Έʒu烄RrAC֐ߑxEʒutZ
  //if(!bArticulated){
		CompJacobian();
		vjrel = Js[1] * solid[1]->v - Js[0] * solid[0]->v;
		UpdateJointState();
	//}
}

// S2̍̂̊ex瑊Αxւ̃RrAvZ
void PHConstraint::CompJacobian(){
	SpatialTransform X[2];
	X[0].r = solid[0]->GetCenterPosition();
	X[0].q = solid[0]->GetOrientation();
	X[1].r = solid[1]->GetCenterPosition();
	X[1].q = solid[1]->GetOrientation();
	Xjrel = Xj[1] * X[1] * X[0].inv() * Xj[0].inv();
	
	Js[0] = Xj[0];
	Js[1] = Xjrel.inv() * Xj[1];
	J[0] = Js[0];
	J[0] *= -1.0;	//p
	J[1] = Js[1];
	
	/*
	//px̍炩quaternion̎Ԕs
	Matrix3d E(
			qjrel.W(),  qjrel.Z(), -qjrel.Y(),
		-qjrel.Z(),  qjrel.W(),  qjrel.X(),
			qjrel.Y(), -qjrel.X(),  qjrel.W());
	E *= 0.5;
	Jqv[0].clear();
	Jqw[0] = E * Jww[0];
	Jqv[1].clear();
	Jqw[1] = E * Jww[1];
	*/
}

/*	ȂΊpvZDA = J * M^-1 * J^T
	As͍S͂瑬xωւ̉e̋\sȂ̂ŁC
	̑Ίp͂S͐玩g̍Sxւ̉e̋\
 */
void PHConstraint::CompResponseMatrix(){
	int i, j;
	A.clear();
	PHRootNode* root[2];
	if(solid[0]->treeNode)
		root[0] = solid[0]->treeNode->GetRootNode();
	if(solid[1]->treeNode)
		root[1] = solid[1]->treeNode->GetRootNode();

	SpatialVector df;
	for(i = 0; i < 2; i++){
		if(solid[i]->IsDynamical()){
			if(solid[i]->treeNode){
				for(j = 0; j < 6; j++){
					if(!constr[j])continue;
					(Vec6d&)df = J[i].row(j);
					solid[i]->treeNode->CompResponse(df, false, false);
					A[j] += J[i].row(j) * solid[i]->treeNode->da;
					int ic = !i;
					//Е̍̂̃c[ɑꍇ͂̉eZ
					if(solid[ic]->treeNode && root[i] == root[ic])
						A[j] += J[ic].row(j) * solid[ic]->treeNode->da;
				}
			}
			else{
				T[i].vv() = J[i].vv() * solid[i]->minv;
				T[i].vw() = J[i].vw() * solid[i]->Iinv;
				T[i].wv() = J[i].wv() * solid[i]->minv;
				T[i].ww() = J[i].ww() * solid[i]->Iinv;
				for(j = 0; j < 6; j++)
					A[j] += J[i].row(j) * T[i].row(j);
			}
		}
	}
	/** ő̑ΊpvfƂ̔䂪epsΊpvfꍇC
		lIs萫̌ƂȂ̂ł͍̐SΏۂ珜O
	 */
	const double eps = 0.01, epsabs = 1.0e-10;
	double Amax = 0.0, Amin;
	for(j = 0; j < 6; j++)
		if(constr[j] && A[j] > Amax)
			Amax = A[j];
	Amin = Amax * eps;
	for(j = 0; j < 6; j++){
		if(!constr[j])continue;
		if(A[j] < Amin || A[j] < epsabs)
			constr[j] = false;
		else
			Ainv[j] = 1.0 / (A[j] + dA[j]);
	}
}

void PHConstraint::SetupLCP(){
	bFeasible = solid[0]->IsDynamical() || solid[1]->IsDynamical();
	if(!bEnabled || !bFeasible)
		return;

	/* O̒lk̂lƂD
	   O̒l̂܂܂lɂƁCS͂ɑ傷ƂۂD
	   ́CLCPLiۂɂ10xj̔őł؂邽߂ƎvD
	   0xNglɗpĂǂC̏ꍇrI̔񐔂vD
	  */
	
	// S鎩Rx̌
	bool con[6];
	SetConstrainedIndex(con);
	for(int i = 0; i < 6; i++){
		if(con[i] && constr[i]){				// pčSꍇ
			 f[i] *= engine->shrinkRate;
			 F[i] *= engine->shrinkRateCorrection;
		}
		else{
			f[i] = 0.0;					// VKɍS or SȂ
			F[i] = 0.0;
		}
		constr[i] = con[i];
	}
	FPCK_FINITE(f.v());

	// ABȀꍇ͂܂
	if(bArticulated)return;

	if(mode == MODE_TORQUE)
		AddMotorTorque();

	// LCP̍W̎Ȋ֐߂̓RrAɍWϊ
	ModifyJacobian();

	dA.clear();
	db.clear();
	CompBias();	// ڕWCol_pɂ␳dA, dbvZ
	
	// LCPAs̑ΊpvZ
	CompResponseMatrix();

	// LCPbxNg
	b = J[0] * solid[0]->v + J[1] * solid[1]->v;
	
	// S͏lɂ鑬xωʂvZ
	SpatialVector fs;
	for(int i = 0; i < 2; i++){
		if(!solid[i]->IsDynamical() || !IsInactive(i))continue;
		if(solid[i]->treeNode){
			(Vec6d&)fs = J[i].trans() * f;
			solid[i]->treeNode->CompResponse(fs, true, false);
		}
		else solid[i]->dv += T[i].trans() * f;
	}

}

void PHConstraint::IterateLCP(){
	if(!bEnabled || !bFeasible || bArticulated)
		return;
	FPCK_FINITE(f.v());

	SpatialVector fnew, df, dfs;
	int i, j;
	for(j = 0; j < 6; j++){
		if(!constr[j])continue;
		fnew[j] = f[j] - Ainv[j] * (dA[j] * f[j] + b[j] + db[j] + J[0].row(j) * solid[0]->dv + J[1].row(j) * solid[1]->dv);
		//fnew[j] = f[j] - (dA[j] * f[j] + b[j] + db[j] + J[0].row(j) * solid[0]->dv + J[1].row(j) * solid[1]->dv);
		/*FPCK_FINITE(AinvJ[0].vv.row(j));
		FPCK_FINITE(AinvJ[1].vv.row(j));
		if (!FPCK_FINITE(fnew.v)){
			FPCK_FINITE(b.v);
			DSTR << AinvJ[0].vv << AinvJ[1].vv;
			DSTR << AinvJ[0].vw << AinvJ[1].vw;
			DSTR << dA.v[j];
			DSTR << std::endl;
			DSTR << "f.v:" << f.v << "b.v:" << b.v << std::endl;
			DSTR << "s0:" << (solid[0]->dv.v) << (solid[0]->dv.w) << std::endl;
			DSTR << "s1:" << (solid[1]->dv.v) << (solid[1]->dv.w) << std::endl;
		}*/
		Projection(fnew[j], j);
		df[j] = fnew[j] - f[j];
		for(i = 0; i < 2; i++){
			if(!solid[i]->IsDynamical() || !IsInactive(i))continue;
			if(solid[i]->treeNode){
				(Vec6d&)dfs = J[i].row(j) * df[j];
				solid[i]->treeNode->CompResponse(dfs, true, false);
			}
			else solid[i]->dv += T[i].row(j) * df[j];
		}
		f[j] = fnew[j];
	}
	//DSTR << f << endl;
}

void PHConstraint::SetupCorrectionLCP(){
	if(!bEnabled || !bFeasible || bArticulated)
		return;
	B.clear();
	CompError();
	
	// velocity updateɂeZ
	B += (J[0] * (solid[0]->v + solid[0]->dv) + J[1] * (solid[1]->v + solid[1]->dv)) * scene->GetTimeStep();
	B *= engine->posCorrectionRate;
		
	// S͏lɂ鑬xωʂvZ
	SpatialVector Fs;
	for(int i = 0; i < 2; i++){
		if(!solid[i]->IsDynamical() || !IsInactive(i))continue;
		if(solid[i]->treeNode){
			(Vec6d&)Fs = J[i].trans() * F;
			solid[i]->treeNode->CompResponse(Fs, true, true);
		}
		else solid[i]->dV += T[i].trans() * F;
	}

	/*double tmp = B[0];
	for(j = 1; j < dim_c; j++)
		if(tmp < B[j])
			tmp = B[j];
	if(tmp > max_error)
		B *= (max_error / tmp);*/
}

void PHConstraint::IterateCorrectionLCP(){
	if(!bEnabled || !bFeasible || bArticulated)
		return;
	
	SpatialVector Fnew, dF, dFs;
	int i, j;
	for(j = 0; j < 6; j++){
		if(!constr[j])continue;
		Fnew[j] = F[j] - /*Ainv[j]*/(1.0 / A[j]) * (B[j] + J[0].row(j) * solid[0]->dV + J[1].row(j) * solid[1]->dV);
		ProjectionCorrection(Fnew[j], j);
		dF[j] = Fnew[j] - F[j];
		for(i = 0; i < 2; i++){
			if(!solid[i]->IsDynamical() || !IsInactive(i))continue;
			if(solid[i]->treeNode){
				(Vec6d&)dFs = J[i].row(j) * dF[j];
				solid[i]->treeNode->CompResponse(dFs, true, true);			
			}
			else solid[i]->dV += T[i].row(j) * dF[j];
		}
		F[j] = Fnew[j];
	}
}

}
