/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#include "Physics.h"
#ifdef USE_HDRSTOP
#pragma hdrstop
#endif

using namespace PTM;
using namespace std;
namespace Spr{;

//----------------------------------------------------------------------------
// PHContactPoint
IF_OBJECT_IMP(PHContactPoint, PHConstraint);

PHContactPoint::PHContactPoint(const Matrix3d& local, PHShapePairForLCP* sp, Vec3d p, PHSolid* s0, PHSolid* s1){
	shapePair = sp;
	pos = p;
	solid[0] = s0, solid[1] = s1;

	Vec3d rjabs[2];
	for(int i = 0; i < 2; i++){
		rjabs[i] = pos - solid[i]->GetCenterPosition();	//̂̒SڐG_܂ł̃xNg
	}
	// local: ڐG_̊֐߃t[ ́Cx@, y,zڐƂ
	Quaterniond qlocal;
	qlocal.FromMatrix(local);
	for(int i = 0; i < 2; i++){
		Xj[i].q = solid[i]->GetOrientation().Conjugated() * qlocal;
		Xj[i].r = solid[i]->GetOrientation().Conjugated() * rjabs[i];
	}
}

PHContactPoint::PHContactPoint(PHShapePairForLCP* sp, Vec3d p, PHSolid* s0, PHSolid* s1){
	shapePair = sp;
	pos = p;
	solid[0] = s0, solid[1] = s1;

	Vec3d rjabs[2], vjabs[2];
	for(int i = 0; i < 2; i++){
		rjabs[i] = pos - solid[i]->GetCenterPosition();	//̂̒SڐG_܂ł̃xNg
		vjabs[i] = solid[i]->GetVelocity() + solid[i]->GetAngularVelocity() % rjabs[i];	//ڐG_ł̑x
	}
	//ڐxNgt[0], t[1] (t[0]͑ΑxxNgɕsɂȂ悤ɂ)
	Vec3d n, t[2], vjrel, vjrelproj;
	n = shapePair->normal;
	vjrel = vjabs[1] - vjabs[0];
	vjrelproj = vjrel - (n * vjrel) * n;		//ΑxxNg@ɒ镽ʂɎˉexNg
	double vjrelproj_norm = vjrelproj.norm();
	if(vjrelproj_norm < 1.0e-10){
		t[0] = n % Vec3d(1.0, 0.0, 0.0);	
		if(t[0].norm() < 1.0e-10)
			t[0] = n % Vec3d(0.0, 1.0, 0.0);
		t[0].unitize();
	}
	else{
		t[0] = vjrelproj / vjrelproj_norm;
	}
	t[1] = t[0] % n;
	Matrix3d Rjabs;
	// ڐG_̊֐߃t[x, yڐCz@Ƃ
	Rjabs.col(0) = n;
	Rjabs.col(1) = t[0];
	Rjabs.col(2) = t[1];
	Quaterniond qjabs;
	qjabs.FromMatrix(Rjabs);
	for(int i = 0; i < 2; i++){
		Xj[i].q = solid[i]->GetOrientation().Conjugated() * qjabs;
		Xj[i].r = solid[i]->GetOrientation().Conjugated() * rjabs[i];
	}
}

void PHContactPoint::SetConstrainedIndex(bool* con){
	con[0] = con[1] = con[2] = true;
	con[3] = con[4] = con[5] = false;
}

double PHContactPoint::correctionSpring = 100.0;
double PHContactPoint::correctionDamper = 10.0;

void PHContactPoint::CompBias(){
	double dtinv = 1.0 / scene->GetTimeStep();
//	db.v.x = 0.1*engine->correctionRate * (-shapePair->depth * dtinv + vjrel.v.x);
	/*	hase	{ 1e-3͈
		depth CorrectionƐڐGsAɂȂ̂ŁCdepth-epsilonŗǂC
		epsilon傫D

		́CڐGʂlp`ŐڐG_4_ꍇȂǁC
		4_Ԃ̗͂̕U܂ɕsψɂȂ]n܂C
		ڐGʂȄԂƍ̏Ԃ̊ԂōׂU邱Ƃ邽߁D
		́C̏ꍇɂڐGʂ̌`ςȂ悤ɂ邱ƂŁC艻ĂD

		2̊Ԃ̐ڐGЂƂ̐ƂĂ킹悤ɂȂΉƎvD	
	*/
#if 0
	double err = (shapePair->depth - 1e-3)*dtinv - 0.2*vjrel.v().x;
	if (err < 0) err = 0;
	if (err){
//		HASE_REPORT
//		DSTR << shapePair->state;
//		DSTR << shapePair->shapePoseW[1].Pos();
//		DSTR << "err: " << err << std::edl;
		db.v().x = -err * engine->velCorrectionRate;
	}
#else
//	const double damper = 100.0, spring = 10.0;
	double tmp = 1.0 / (correctionDamper + correctionSpring * scene->GetTimeStep());
	dA[0] = tmp * dtinv;
	db[0] = -correctionSpring * (shapePair->depth - 1e-3) * tmp;
#endif
}

void PHContactPoint::Projection(double& f, int k){
	static double flim;
	if(k == 0){	//R >= 0̐
//		f = min(1.0, f);	// fp̋̍Bō폜ׂ
		f = max(0.0, f);
		flim = 0.5 * (shapePair->shape[0]->material.mu0 + shapePair->shape[1]->material.mu0) * f;	//őÎ~C
	}
	else{
		//|C| <= őÎ~C̐
		//	EC͂̊eőÎ~CĂ͂͒\̂Ŗ{͂B
		//	EÎ~CƓClłȂƈȂB
		//CW͗҂̐Î~CW̕ςƂ
		f = min(max(-flim, f), flim);
	}
}

void PHContactPoint::CompError(){
	const double eps = 0.0;
	//Փ˔ASY̓sACorrectionɂĊSɍ̂Ă܂͍̂̂
	//덷eps
	B.v().x = min(0.0, -shapePair->depth + eps);
}

void PHContactPoint::ProjectionCorrection(double& F, int k){
	if(k == 0){	//R >= 0̐
		F = max(0.0, F);
	}
	else if(k == 1 || k == 2)
		F = 0;
}

}
