/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef PHPATHJOINT_H
#define PHPATHJOINT_H

#include <SprPhysics.h>
#include <Physics/PHSpatial.h>
#include <Physics/PHConstraint.h>

namespace Spr{;

/*	2̍̊Ԃ̑ΈʒuEpԂ֐
		[p; q] = f(s)
	ƂDp͑Έʒu, q͑quaternionCs͈ʉW\킷XJ[D
	
		[p'; q'] = (df/ds)(s)s'D
	quaternion̎Ԕpx𓾂sE(q)Ƃ
		[v; w] = [I O; O E(q(s))](df/ds)(s)s' =: J(s)s'
	̎ss'܂ΑΑx[v; w]܂D
	Š`ɕςD6xNgJ(s)͑Αx̌\킵Ă̂ŁC
	ɒxNgsxNgɎsJ~(s)ƂƁCS
		J~(s)[v; w] = 0

	CorrectiońC
	velocity updatȇΈʒuɍł߂Oim(f)̓_擾C̓_ɍ킹D
	{̍S[p; q] in im(f)`Ȃ̂ňȂD
 */

/// ֐߂̋ÕL[t[C⊮ĕԂ胄RrAvZ肷NX
struct PHPathPointWithJacobian : public PHPathPoint{
	Matrix6d	J;
};

class PHPath : public SceneObject, public PHPathIfInit, public std::vector<PHPathPointWithJacobian>{
	bool bReady;
	bool bLoop;	//[-pi, pi]̖]֐
	iterator Find(double &s);
public:
	OBJECTDEF_ABST(PHPath, SceneObject);
	virtual bool GetDesc(void* desc)const;
	virtual void SetDesc(const void* desc);

	virtual void SetLoop(bool OnOff = true){bLoop = OnOff;}
	virtual bool IsLoop(){return bLoop;}
	void AddPoint(double s, const Posed& pose);
	void CompJacobian();
	void GetPose(double s, Posed& pose);
	void GetJacobian(double s, Matrix6d& J);
	void Rollover(double& s);
	PHPath(const PHPathDesc& desc);
};

class PHPathJointNode : public PHTreeNode1D, public PHPathJointNodeIfInit{
public:
	OBJECTDEF(PHPathJointNode, PHTreeNode1D);
	PHPathJoint* GetJoint(){return DCAST(PHPathJoint, PHTreeNode1D::GetJoint());}
	virtual void CompJointJacobian();
	virtual void CompJointCoriolisAccel();
	virtual void CompRelativePosition();
	virtual void CompRelativeVelocity();
	virtual void UpdateJointPosition(double dt);
	PHPathJointNode(const PHPathJointNodeDesc& desc = PHPathJointNodeDesc()){}
};

class PHPathJoint : public PHJoint1D, public PHPathJointIfInit{
	OBJECTDEF(PHPathJoint, PHJoint1D);
	UTRef<PHPath> path;
public:
	//virtual PHConstraintDesc::ConstraintType GetConstraintType(){return PHConstraintDesc::PATHJOINT;}
	virtual PHTreeNode* CreateTreeNode(){
		return DBG_NEW PHPathJointNode();
	}
	virtual void SetPosition(double pos){position[0] = pos;}
	virtual bool AddChildObject(ObjectIf* o);
	virtual void ModifyJacobian();
	virtual void CompBias();
	virtual void UpdateJointState();
	//virtual void CompError(double dt);
	//virtual void ProjectionCorrection(double& F, int k);
	PHPathJoint(const PHPathJointDesc& desc = PHPathJointDesc());
};

}

#endif
