/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef PHPENALTYENGINE_H
#define PHPENALTYENGINE_H

#include <Physics/PHContactDetector.h>

namespace Spr{;

class PHShapePairForPenalty : public CDShapePair{
public:
	///@name	R͂̌vZ
	//@{
	float area;							///<	̖ʐ
	Vec3f reflexSpringForce;			///<	΂˂ɂR
	Vec3f reflexDamperForce;			///<	_p[ɂR
	Vec3f reflexSpringTorque;			///<	΂˂ɂRgN(commonPointn)
	Vec3f reflexDamperTorque;			///<	_p[ɂRgN(commonPointn)
	Vec3f reflexForcePoint;				///<	R͂̍p_(commonPointn)
	//@}

	///@name	ČvZ
	//@{
	Vec3f transFrictionBase[2];			///<	iÎ~Cpol̒[_([Jt[n)
	float rotSpring;					///<	]΂˂̐L(WA)
	Vec3f dynaFric;						///<	C
	Vec3f dynaFricMom;					///<	C͂̃[g(commonPointn)
	Vec3f frictionForce;				///<	C
	Vec3f frictionTorque;				///<	CgN(C͂̍p_  (reflexForcePoint+commonPoint)n)
	enum FrictionState{ STATIC, DYNAMIC };
	FrictionState frictionState;		///<	C̏
	//@}

	void Clear();
	
	///@name	΍Wnł̓ǂݏo
	//@{
	///	olɂR͂̍p_
	Vec3f GetReflexForcePoint(){ return reflexForcePoint + commonPoint;}
	///	olɂR
	Vec3f GetReflexSpringForce(){ return reflexSpringForce;}
	///	_pɂR
	Vec3f GetReflexDamperForce(){ return reflexDamperForce;}
	///	R
	Vec3f GetReflexForce(){ return reflexSpringForce + reflexDamperForce;}

	///	C
	Vec3f GetFrictionForce(){ return frictionForce; }
	///	CgN
	Vec3f GetFrictionTorque(){ return frictionTorque + (GetReflexForcePoint()^frictionForce); }
	//@}
};

class PHPenaltyEngine;
class PHSolidPairForPenalty : public PHSolidPair<PHShapePairForPenalty, PHPenaltyEngine>{
public:
	typedef PHSolidPair<PHShapePairForPenalty, PHPenaltyEngine> base_type;
	typedef base_type::shapepair_type shapepair_type;
	typedef base_type::engine_type engine_type;

	Vec3f cocog;					///<	2̂̏dS̒_(Όn)
	Vec3f reflexForce;				///<	R
	Vec3f reflexTorque;				///<	R͂ɂgN(cocogn)
	Vec3f frictionForce;			///<	C
	Vec3f frictionTorque;			///<	C͂ɂgN(cocogn)
	float convertedMass;			///<	̂̑g̊Z
	float area;						///<	ڐGʐ

	virtual void OnDetect(shapepair_type* sp, engine_type* engine, unsigned ct, double dt);	///< mꂽƂ̏

	void Setup(unsigned int ct, double dt);
	void GenerateForce();

	///@name	Όnł̓ǂݏo
	//@{
	///	R
	Vec3f GetReflexForce(){ return reflexForce; }
	///	R͂̃gN
	Vec3f GetReflexTorque(){ return reflexTorque + (cocog^reflexForce); }
	///	C
	Vec3f GetFrictionForce(){ return frictionForce; }
	///	C͂̃gN
	Vec3f GetFrictionTorque(){ return frictionTorque + (cocog^frictionForce); }
	//@}
	
	///	͂̍ől𐧖񂷂D
	void LimitForces(){
		bool b = false;
		b |= LimitForce(reflexForce);
		b |= LimitForce(frictionForce);
		b |= LimitForce(reflexTorque);
		b |= LimitForce(frictionTorque);
		if (b){
			reflexForce = frictionForce = reflexTorque = frictionTorque = Vec3f();
		}
	}
protected:
	bool LimitForce(Vec3f& f){
		float sq = f.square();
		const float MF = 200000;
		if (!(sq < MF*MF)){
			return true;
		}
		return false;
	}
	void CalcReflexForce(PHShapePairForPenalty* sp, CDContactAnalysis* analyzer);
	void CalcTriangleReflexForce(PHShapePairForPenalty* cp, Vec3f p0, Vec3f p1, Vec3f p2, Vec3f cog, Vec3f vel, Vec3f angVel);
	void CalcFriction(PHShapePairForPenalty* cp);
};

class PHPenaltyEngine : public PHContactDetector<PHShapePairForPenalty, PHSolidPairForPenalty, PHPenaltyEngine>{
	OBJECTDEF_NOIF(PHPenaltyEngine, PHEngine);
public:
	int GetPriority() const {return SGBP_PENALTYENGINE;}
	virtual void Step();
};

}	//	namespace Spr
#endif
