/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef PHSCENE_H
#define PHSCENE_H
#include <Physics/SprPHScene.h>
#include <Foundation/Scene.h>
#include <Physics/PHEngine.h>
#include <Physics/PHSolid.h>

namespace Spr {;

class CDShape;
struct CDShapeDesc;
class PHSolidContainer;
class PHPenaltyEngine;
class PHConstraintEngine;
class PHGravityEngine;
struct PHConstraintDesc;
typedef PHConstraintDesc PHJointDesc;

/**	V[Ot̃gbvm[hDE_D
	_ƃV[Ot̊֌W[߁C
	_ႤƃV[Ot͂̂܂܎gpłȂD
	V[Ot́CƂ΃_OpenGLȂCdisplayList
	Ă邵CD3DȂ ID3DXMeshĂD
	OpenGL̃V[OtD3Dɕϊ邽߂ɂ́CxDocument
	Z[uāCD3D`Ń[hȂ΂ȂȂD	*/
class SPR_DLL PHScene : public Scene, public PHSceneIfInit, public PHSceneDesc{
	OBJECTDEF(PHScene, Scene);
	friend class PHConstraint;
public:
	PHEngines engines;
protected:
	PHSolidContainer*	solids;
	PHPenaltyEngine*	penaltyEngine;
	PHConstraintEngine* constraintEngine;
	PHGravityEngine*	gravityEngine;
public:
	///	RXgN^
	PHScene(const PHSceneDesc& desc=PHSceneDesc());
	void Init();
	///	fXgN^
	~PHScene(){}

	PHConstraintEngine* GetConstraintEngine();
	PHGravityEngine*	GetGravityEngine();
	PHSolidIf* CreateSolid(const PHSolidDesc& desc = PHSolidDesc());
	int NSolids() const;
	PHSolidIf** GetSolids();
	CDShapeIf* CreateShape(const IfInfo* ii, const CDShapeDesc& desc);
	PHJointIf* CreateJoint(PHSolidIf* lhs, PHSolidIf* rhs, const IfInfo* ii, const PHJointDesc& desc);
	int NJoints() const;
	PHJointIf* GetJoint(int i);
	PHRootNodeIf* CreateRootNode(PHSolidIf* root, const PHRootNodeDesc& desc = PHRootNodeDesc());
	int NRootNodes() const;
	PHRootNodeIf* GetRootNode(int i);
	PHTreeNodeIf* CreateTreeNode(PHTreeNodeIf* parent, PHSolidIf* child, const PHTreeNodeDesc& desc = PHTreeNodeDesc());
	PHGearIf*	CreateGear(PHJoint1DIf* lhs, PHJoint1DIf* rhs, const PHGearDesc& desc = PHGearDesc());
	int NGears() const;
	PHGearIf* GetGear(int i);
	PHPathIf*	CreatePath(const PHPathDesc& desc = PHPathDesc());
	int NPaths() const;
	PHPathIf* GetPath(int i);
	void SetContactMode(PHSolidIf* lhs, PHSolidIf* rhs, PHSceneDesc::ContactMode = PHSceneDesc::MODE_LCP);
	void SetContactMode(PHSolidIf** group ,size_t length, PHSceneDesc::ContactMode mode = PHSceneDesc::MODE_LCP);
	void SetContactMode(PHSolidIf* solid, PHSceneDesc::ContactMode = PHSceneDesc::MODE_LCP);
	void SetContactMode(PHSceneDesc::ContactMode mode = PHSceneDesc::MODE_LCP);
	virtual int GetNumIteration();
	virtual void SetNumIteration(int n);
	PHSdkIf* GetSdk();

	/// ϕXebvԂ
	double GetTimeStep()const{return timeStep;}
	/// ϕXebvݒ肷
	void SetTimeStep(double dt);
	/// JEgԂ
	unsigned GetCount()const{return count;}
	/// JEgݒ肷
	void SetCount(unsigned c){count = c;}
	///	V[̎i߂ ClearForce(); GenerateForce(); Integrate(); Ɠ
	void Step();
	///	V[̎i߂i͂̃NAj
	void ClearForce();
	///	V[̎i߂i͂̐j
	void GenerateForce();
	///	V[̎i߂i͂ƑxϕāCxƈʒuXVj
	void Integrate();
	///	V[ɂD
	void Clear();

	virtual void SetGravity(const Vec3d& accel);
	virtual Vec3f GetGravity();

	ObjectIf* CreateObject(const IfInfo* info, const void* desc);
	virtual size_t NChildObject() const;
	virtual ObjectIf* GetChildObject(size_t pos);
	virtual bool AddChildObject(ObjectIf* o);
	virtual bool DelChildObject(ObjectIf* o);

	friend class PHSolid;
	friend class Object;
	ACCESS_DESC_STATE(PHScene);
};

}
#endif
