/*
 *  Copyright (c) 2003-2006, Shoichi Hasegawa and Springhead development team 
 *  All rights reserved.
 *  This software is free software. You can freely use, distribute and modify this 
 *  software. Please deal with this software under one of the following licenses: 
 *  This license itself, Boost Software License, The MIT License, The BSD License.   
 */
#ifndef PHTREENODE_H
#define PHTREENODE_H

#include <SprPhysics.h>
#include <Physics/PHSpatial.h>

namespace Spr{;

///	c[m[h̊{NX
class PHRootNode;
class PHTreeNode : public SceneObject, public PHTreeNodeIfInit, public UTTreeNode<PHTreeNode>, public PHTreeNodeDesc{
public:
	OBJECTDEF_ABST(PHTreeNode, SceneObject);
	
	virtual bool AddChildObject(ObjectIf* o);
	virtual size_t NChildObject();
	virtual ObjectIf* GetChildObject(size_t i);
	virtual void Enable(bool bEnable = true);
	
	bool		Includes(PHTreeNode* node);		///< ȉɃm[hnode邩
	PHTreeNode*	FindBySolid(PHSolid* solid);	///< ȉ̃c[solidQƂĂm[hT
	PHTreeNode* FindByJoint(PHJoint* joint);	///< ȉ̃c[jointQƂĂm[hT
	void		RemoveGearNode();
	void		Prepare(PHScene* scene, PHConstraintEngine* engine);	///< ċAIsceneengineZbg

	PHTreeNode();

	virtual PHSolid*	GetSolid(){return joint->solid[1];}
	virtual PHRootNode*	GetRootNode(){return GetParent()->GetRootNode();}
	virtual void ResetGearNode();
	virtual void CompAccel()=0;
	virtual void CompAccelDiff(bool bUpdate, bool bImpulse)=0;
	/// ̃m[hɉS͂̕ωʂ瑼̃m[h̑xωʂւ̉evZDLCPŎgp
	virtual void CompResponse(const SpatialVector& df, bool bUpdate, bool bImpulse);
	virtual void CompBiasForceDiff(bool bUpdate, bool bImpulse)=0;
	virtual void SetupLCP();
	virtual void IterateLCP();
	virtual void InitArticulatedInertia();
			void InitArticulatedBiasForce();
	virtual void CompArticulatedInertia();			///< Articulated InertiǎvZ
	virtual void CompArticulatedBiasForce();		///< Articulated Bias ForcěvZ
	virtual	void CompSpatialTransform();			///< m[hԂ̍Wϊ̌vZ
	virtual void CompCoriolisAccel();
	virtual void CompJointJacobian(){}				///< ֐߃RrAvZ
	virtual void CompJointCoriolisAccel(){}			///< RỈxvZ	
	virtual void AccumulateInertia(){}				///< Iaem[hIaɐςݏグ
	virtual void AccumulateBiasForce(){}			///< Zaem[hZaɐςݏグ
			void ResetCorrectionVarsRecursive();
	virtual void ResetCorrectionVars();				///< Correction̕ϐ̏
	virtual void UpdateVelocity(double dt);			///< ̂̑x̍XV
	virtual void UpdatePosition(double dt);			///< ̂̈ʒu̍XV
	virtual void UpdateJointVelocity(double dt){}	///< ֐ߑx̍XV
	virtual void UpdateJointPosition(double dt){}	///< ֐߈ʒu̍XV
	virtual void CompRelativeVelocity(){}			///< ֐ߑx獄̊ԑΑxvZ
	virtual void CompRelativePosition(){}			///< ֐߈ʒu獄̊ԑΈʒuvZ
	
	//int			id;						///< IDD[gm[h0depth-firstŊ蓖ĂD
	PHJoint*	joint;						///< em[hƂ̃m[hƂȂS
	//bool		opposite;					///< c[m[h̐eq֌WƍS̐eq֌Wi\PbgĂ̂ejtȂtrue

	SpatialMatrix			I;				///< Articulated Inertia
	SpatialVector			Z;				///< Articulated Bias Force
	SpatialVector			dZ;				///< S͂̕ωɂZ̕ω
	SpatialVector			c;				///< RIx
	SpatialVector			cj;				///< ֐ߑxɂRIx

	SpatialTransform		Xcp, Xcj;
	SpatialMatrix			XIX, XtrIJ_JIJinv_Jtr, XtrIJ_JIJinv_JtrIX;
	SpatialVector			a, da;			///< xƂ̕ω
	//SpatialVector			v, dv;			///< xƂ̕ω
	SpatialVector			ap, Ic, ZplusIc, XtrZplusIc;

	PHScene*	scene;
	PHConstraintEngine* engine;
};

class PHRootNode : public PHTreeNode, public PHRootNodeIfInit{
public:
	OBJECTDEF(PHRootNode, PHTreeNode);

	virtual bool AddChildObject(ObjectIf* o);
	virtual size_t NChildObject();
	virtual ObjectIf* GetChildObject(size_t i);
	
	void SetupABA();
	void SetupCorrectionABA();
	
	virtual PHSolid* GetSolid(){return solid;}
	virtual PHRootNode* GetRootNode(){return this;}
	virtual int	 GetDof(){ return 6; }
	virtual void CompArticulatedInertia();
	virtual void CompArticulatedBiasForce();
	virtual void CompAccel();
	virtual void CompAccelDiff(bool bUpdate, bool bImpulse){}
	virtual void CompBiasForceDiff(bool bUpdate, bool bImpulse);
	virtual void UpdateVelocity(double dt);
	virtual void UpdatePosition(double dt);
	
	PHSolid*	solid;		///< [gm[h̍

	PHRootNode(const PHRootNodeDesc& desc = PHRootNodeDesc()){
		solid = NULL;
	}
protected:
	SpatialMatrix		Iinv;
};

///	NRx̊֐߂̊{NX
template<int NDOF>
class PHTreeNodeND : public PHTreeNode{
public:
	typedef	PTM::TVector<NDOF, double>	VecNd;
	typedef PTM::TMatrixCol<6, NDOF, double> Matrix6Nd;
	typedef PTM::TMatrixCol<NDOF, NDOF, double> MatrixNd;
	
	PHJointND<NDOF>* GetJoint(){return (PHJointND<NDOF>*)DCAST(PHJoint , joint);}

	/// ABA֌W̊֐
	virtual void ResetGearNode();
	virtual void CompCoriolisAccel();
	virtual void CompSpatialTransform();
	virtual void InitArticulatedInertia();
	virtual void AccumulateInertia();
	virtual void AccumulateBiasForce();
	virtual void CompJointJacobian();
	virtual void CompAccel();
	virtual void ResetCorrectionVars();
	virtual void CompAccelDiff(bool bUpdate, bool bImpulse);
	virtual void CompBiasForceDiff(bool bUpdate, bool bImpulse);
	virtual void UpdateJointVelocity(double dt);
	virtual void UpdateJointPosition(double dt);

	/// LCP֌W̊֐
	virtual void CompResponse(const VecNd& tau, bool bUpdate);
			void CompResponseMatrix();
	virtual void ModifyJacobian();
	virtual void SetupLCP();
	virtual void IterateLCP();
	virtual void CompBias()=0;
	virtual void Projection(double& f, int k)=0;

	void AddGear(PHGear* gear, PHTreeNodeND<NDOF>* child);
	PHGear*			gear;				///< MAւ̎QƁD̃MAĐem[hƘA
	PHTreeNodeND<NDOF>*	gearNode;			///< MAm[hւ̎QƁDNULL:AȂ, this:MAgCŏi
	std::vector<PHTreeNodeND<NDOF>*> gearChildren;	///< ggearNodȅꍇCAĂm[h̔z
	PHTreeNodeND<NDOF>*	parentND;
	
	PHTreeNodeND();
//protected:
	bool			constr[NDOF];			///< ABAƂĊeRxS邩
	Matrix6Nd		J;						///< ֐ߍW瑊Αxւ̃RrA
	Matrix6Nd		IJ, XtrIJ, J_JIJinv, IJ_JIJinv, XtrIJ_JIJinv;
	MatrixNd		JIJ, JIJinv;
	VecNd	JtrZplusIc;
	VecNd	accel, daccel, dtau, vel;	///< ֐߉xiƕωʁjC֐߃gN̕ωʁC֐ߑx

	SpatialTransform	Xcg;			///< MAgC̐em[hƂ̊Ԃ̍Wϊ
	SpatialMatrix	sumXIX;
	SpatialVector	sumXtrZplusIc, sumXtrdZ;
	Matrix6Nd		sumXtrIJ, sumXtrIJ_sumJIJinv;
	MatrixNd		sumJIJ, sumJIJinv;
	VecNd			sumJtrZplusIc, sumJtrdZ, sumtorque, JtrdZ;
	

	VecNd A, Ainv, dA, b, db, f;	///< ֐ߍWLCP̂߂̕ϐ
	MatrixNd Jq;					///< ֐ߍWLCPɂ֐ߑxSϐւ̃RrA
};

///	1Rx̊֐
class PHTreeNode1D;
class PHTreeNode1D : public PHTreeNodeND<1>, public PHTreeNode1DIfInit{
public:
	OBJECTDEF_ABST(PHTreeNode1D, PHTreeNode);
	
	PHJoint1D*		GetJoint(){return DCAST(PHJoint1D, joint);}
	virtual void	CompBias();
	virtual void	Projection(double& f, int k);
	
	PHTreeNode1D();
};

}

#endif
